-----------------------------------------------------------------------------
--
--		Standard digit setup
--		
--		v0.5: 	Basic digit
--		v0.6: 	Named all objects and put them into layers
--		v0.7: 	Individual digit control objects
--				Stretch bones inside hand
--				Position attributes on digits
--		v0.8:	Support for short digits (thumbs and toes)
--		v0.9:	Toes
--		v1.0:	Stretch bones for toes
--		v1.1:	Minor change of naming
--		v1.2:	Finger controller moved from arm to this script
--
-----------------------------------------------------------------------------
global SSR_part

struct SSR_part
(
	function rigDigit charName locatorPrefix digit side limbControl digitParent stretchParent digitType boneSize:1 debug:false =
	(
		local helperList = $'Locator_*'
		local posFingRoot, posFingA, posFingB, posFingC, posFingD = undefined
		local prefix = (charName + "_" + side + "_" + digit + digitType + "_")
		local shortDigit = false
		local stretchRoot = false
		
		-- Search for locators running down the length of current digit
		-- Typical locatorprefix is "Locator_digit_R_Thumb"
		if helperList != false then for h in helperList do
		(
			if matchPattern h.name pattern:(locatorPrefix + digit + "Root") == true then posFingRoot = h.pos
			if matchPattern h.name pattern:(locatorPrefix + digit + "A") == true then posFingA = h.pos
			if matchPattern h.name pattern:(locatorPrefix + digit  + "B") == true then posFingB = h.pos
			if matchPattern h.name pattern:(locatorPrefix + digit  + "C") == true then posFingC = h.pos
			if matchPattern h.name pattern:(locatorPrefix + digit  + "D") == true then posFingD = h.pos
		)
		
		-- If one of these helpers isn't there, we will exit
		if posFingA == undefined or posFingB == undefined or posFingC == undefined
		then (
			if debug == true then SSR_gen.verbose ("Missing locators for " + digit + ".")
			return false
		) else (
			if debug == true then SSR_gen.verbose ("All locators are found, now creating " + side + " " + digit + ".")
		)
		
		-- If we don't find a fourth digit locator, the digit will be a short thumb type
		if posFingD == undefined then 
		(
			shortDigit = true
			if debug == true then SSR_gen.verbose ("Creating a short digit.")
		)

		-- If we find a root position locator, we create a stretching root bone
		if posFingRoot != undefined then stretchRoot = true
		
		
	-----------------------------------------------------------------------------
	--
	--		Create digit bones and link them up
	--
	-----------------------------------------------------------------------------
		
		--local boneSize = (distance posFingA posFingC) / 4
		local zAxis = normalize ( cross (posFingA-posFingB) (posFingB-posFingC) )
		
		local boneA		= SSR_rig.makeBone (prefix + "skinBone_A") posFingA posFingB zAxis boneSize
		local boneB		= SSR_rig.makeBone (prefix + "skinBone_B") posFingB posFingC zAxis boneSize
		if shortDigit == false then
		(
			local boneC		= SSR_rig.makeBone (prefix + "skinBone_C") posFingC posFingD zAxis boneSize
			local boneEnd	= SSR_rig.makeBone (prefix + "bone_End") posFingD (posFingD - [0,0,boneSize]) zAxis boneSize
			SSR_rig.objAlign boneEnd boneC
		) else (
			local boneEnd	= SSR_rig.makeBone (prefix + "bone_End") posFingC (posFingC - [0,0,boneSize]) zAxis boneSize
			SSR_rig.objAlign boneEnd boneB
		)
			
		boneB.parent = boneA
		
		if shortDigit == false then
		(
			boneC.parent = boneB
			boneEnd.parent = boneC
		) else (
			boneEnd.parent = boneB
		)
		
		-- !! Something is wrong with a selection of the bones, usually on the left, but why?
		-- They end up with a rotation of -180, 180, -180 relative to their parent
		
		--if side == "L" then zAxis = - zAxis
		--superDrawVector (zAxis*4) posFingB
			
	-----------------------------------------------------------------------------
	--
	--		digit control object
	--
	-----------------------------------------------------------------------------
		
		local digitControl = SSR_rig.pinCTL boneA.pos (-boneSize*3) boneSize
		digitControl.name = (charName + "_" + side + "_" + digitType + "_" + digit + "_CTL")
		SSR_rig.objAlign digitControl boneA
		
		setTransformLockFlags digitControl #{7..9}
		
		boneA.parent = digitControl
		digitControl.parent = digitParent
		
		-- Freeze rotation with helper objects
		local controlNull = SSR_rig.nullTransform digitControl
		controlNull.name = prefix + "controlNull"
		local boneBNull = SSR_rig.nullTransform boneB
		boneBNull.name = prefix + "bone_B_null"
		
		if shortDigit == false then
		(
			local boneCNull = SSR_rig.nullTransform boneC
			boneCNull.name = prefix + "bone_C_null"
			local boneCNullNull = SSR_rig.nullTransform boneCNull
		)
		
		-- !! Something is wrong with the orientation of the base bone
		-- Just to be sure, we null out some extra rotations
		digitControl.rotation.X_rotation.controller.value = 0;
		digitControl.rotation.Y_rotation.controller.value = 0;
		digitControl.rotation.Z_rotation.controller.value = 0;
		
		-- !! Also, a temp fix for the occasional weird rotations on our second bone...
		boneB.rotation.X_rotation.controller.value = 0;
		boneB.rotation.Y_rotation.controller.value = 0;
		boneB.rotation.Z_rotation.controller.value = 0;
		
		if shortDigit == false then
		(
			boneC.rotation.X_rotation.controller.value = 0;
			boneC.rotation.Y_rotation.controller.value = 0;
			boneC.rotation.Z_rotation.controller.value = 0;
		)
		
	-----------------------------------------------------------------------------
	--
	--		Stretch bones inside of limb to improve deformation to digits
	--		(This should be made optional)
	--
	-----------------------------------------------------------------------------

		if stretchRoot == true then
		(		
			-- Create and align our bones
			local stretchPrefix		= charName + "_" + side + "_" +  digit + digitType + "_stretch_"
			local stretchBone 		= SSR_rig.makeBone (stretchPrefix + "skinBone") posFingRoot posFingA zAxis boneSize
			local stretchBoneEnd	= SSR_rig.makeBone (stretchPrefix + "bone_End") posFingA (posFingA - [0,0,boneSize]) zAxis boneSize
			SSR_rig.objAlign stretchBoneEnd stretchBone
			stretchBoneEnd.parent = stretchBone
			
			-- Constraint the stretchBone to follow and stretch after our digit controller
			stretchBone.boneFreezeLength = false
			SSR_rig.posCon stretchBoneEnd digitControl
			stretchLookCon = SSR_rig.lookCon stretchBone digitControl
			
			stretchLookCon.upnode_ctrl = 1
			stretchLookCon.upNode_world = false
			stretchLookCon.pickUpNode = stretchParent	
			stretchBone.parent = stretchParent
		)
		
	-----------------------------------------------------------------------------
	--
	--		digit attributes
	--
	-----------------------------------------------------------------------------
		
		global superdigitCa
		
		-- Build digit attributes
		caExe   = "superdigitCa = attributes " + (toLower digit) + "_transforms\n"
		caExe += "(\n"
		caExe += "	parameters params rollout:digitR\n"
		caExe += "	(\n"
		caExe += "		digit_bend		type:#float ui:digit_bend_sp\n"
		if shortDigit == false then caExe += "		digit_curl		type:#float ui:digit_curl_sp\n"
		caExe += "		digit_press	type:#angle ui:digit_press_sp\n"
		caExe += "		digit_splay	type:#angle ui:digit_splay_sp\n"
		caExe += "		digit_twist	type:#angle ui:digit_twist_sp\n"
		caExe += "		X_Pos			type:#float ui:posx_sp\n"
		caExe += "		Y_Pos			type:#float ui:posy_sp\n"
		caExe += "		Z_Pos			type:#float ui:posz_sp\n"
		caExe += "	)\n"
		caExe += "	rollout digitR (\"" + digit + " Transforms\")\n"
		caExe += "	(\n"
		-- Bends and curls (rotates)
		caExe += "		spinner digit_bend_sp 	\"Bend \" 	range:[-130, 130, 0]\n"
		if shortDigit == false then caExe += "		spinner digit_curl_sp	\"Curl \" 	range:[-130, 130, 0]\n"
		caExe += "		spinner digit_press_sp \"Press \" 	range:[-130, 130, 0]\n"
		caExe += "		spinner digit_splay_sp	\"Splay \" 	range:[-130, 130, 0]\n"
		caExe += "		spinner digit_twist_sp \"Twist \" 	range:[-130,130, 0]\n"
		-- Translates
		caExe += "		spinner posx_sp 	\"Translate X \" 	range:[-1000, 1000, 0] offset:[0,5]\n"
		caExe += "		spinner posy_sp		\"Translate Y \" 	range:[-1000, 1000, 0]\n"
		caExe += "		spinner posz_sp 	\"Translate Z \" 	range:[-1000, 1000, 0]\n"
		caExe += "	)\n"
		caExe +=")\n"

		-- Add custom attributes to the digit controller
		local digitMod = 'PEN_Attribute_Holder 2'()
		addModifier digitControl digitMod

		local handMod = limbControl.modifiers[1]
		
		execute caExe
		custAttributes.add handMod superdigitCa
		custAttributes.add digitMod superdigitCa
		
		-- Bend connection
		execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.digit_bend.controller = " + \
					"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.digit_bend.controller = " + \
					"$'" + digitControl.name + "'.rotation.controller.Z_rotation.controller"	)
		
		if shortDigit == false then
		(
			-- Curl connections
			execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.digit_curl.controller = " + \
						"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.digit_curl.controller = " + \
						"$'" + boneB.name + "'.rotation.controller.Z_rotation.controller = " + \
						"$'" + boneC.name + "'.rotation.controller.Z_rotation.controller"	)
						
			-- Press connections
			execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.digit_press.controller = " + \
						"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.digit_press.controller = " + \
						"$'" + boneCNull.name + "'.rotation.controller.Z_rotation.controller"	)	
		) else (
			-- Press connections
			execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.digit_press.controller = " + \
						"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.digit_press.controller = " + \
						"$'" + boneB.name + "'.rotation.controller.Z_rotation.controller"	)	
		)
						
		-- Splay connection
		execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.digit_splay.controller = " + \
					"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.digit_splay.controller = " + \
					"$'" + digitControl.name + "'.rotation.controller.Y_rotation.controller"	)
		
		-- Twist connection
		execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.digit_twist.controller = " + \
					"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.digit_twist.controller = " + \
					"$'" + digitControl.name + "'.rotation.controller.X_rotation.controller"	)			
					
		-- Translate connections
		execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.X_Pos.controller = " + \
					"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.X_Pos.controller = " + \
					"$'" + digitControl.name + "'.position.controller.X_position.controller"	)		

		execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.Y_Pos.controller = " + \
					"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.Y_Pos.controller = " + \
					"$'" + digitControl.name + "'.position.controller.Y_position.controller"	)	

		execute(	"$'" + digitControl.name + "'.modifiers[1]." + digit + "_transforms.Z_Pos.controller = " + \
					"$'" + limbControl.name + "'.modifiers[1]." + digit + "_transforms.Z_Pos.controller = " + \
					"$'" + digitControl.name + "'.position.controller.Z_position.controller"	)	
		
		return digitControl
	),

	function rigFingers charName side:"R" boneSize:1 =
	(
		-- Primary check, if there are no finger locators, we exit
		local locatorPrefix = "Locator_Finger_" + side + "_"
		local helperList = SSR_loc.collectLocators charName (locatorPrefix + "*")
		if helperList == false then 
		(
			SSR_gen.verbose ("! Could not find and finger locators for " + side + " side.")
			return false
		)
		
		-- If there is no hand bone, we cannot continue
		local handBone = undefined
		local handBones = $'*_skinBone_Hand'
		
		if handBones.count != 0 then 
		(
			for b in handBones do
			(
				if matchPattern b.name pattern:(charName + "_" + side + "_arm_skinBone_Hand") == true then handBone = b
			)
		)
		if handBone == undefined then
		(
			SSR_gen.verbose ("! Could not find " + side + " Hand Bone, unable to create fingers.")
			return false
		)
		
		-- Find existing hand rotation controller
		local handRotControl = undefined 
		handRotControl = execute ("$'" + charName + "_" + side + "_hand_FK_CTL'")
		if handRotControl == undefined then handRotControl = execute ("$'" + charName + "_" + side + "_hand_Rot_CTL'")
		if handRotControl == undefined then
		(
			SSR_gen.verbose ("! Could not find " + side + " Hand Rotation Controller, unable to create fingers.")
			return false
		)
		
		-- Create finger controller
		local handControl = SSR_rig.ringCTL (handBone.pos) (boneSize*4) alignAxis:"Z"
		handControl.name = charName + "_" + side + "_fingers_CTL"
		handControl.parent = handRotControl
		SSR_rig.objAlign handControl handRotControl
		if side == "R" then in coordSys local move handControl [boneSize*4,0,boneSize*5]
		else in coordSys local move handControl [boneSize*4,0,-boneSize*5]
			
		penMod_Fingers = 'PEN_Attribute_Holder 2'()
		addModifier handControl penMod_Fingers
		setTransformLockFlags handControl #{1..9}
		
	-----------------------------------------------------------------------------
	--
	--		Create fingers
	--
	-----------------------------------------------------------------------------
		
		-- Let's look for finger locators
		local fingerControllers = #(handControl)
		local locatorPrefix = "Locator_Finger_" + side + "_"
		local helperList = $'Locator_Finger*'
		local fingerNames = #("Thumb", "Index", "Middle", "Ring", "Pinkie")
		
		-- Search for the first A locator on thumb, index, middle, ring and pinkie
		-- If it exists, we start creating the finger
		for h in helperList do
		(
			for n in fingerNames do
			(
				if matchPattern h.name pattern:(locatorPrefix + n + "A") == true then
				(	
					local fingerControl = SSR_part.rigDigit charName locatorPrefix n side handControl handBone handBone "Finger" boneSize:boneSize debug:false
					append fingerControllers fingerControl
				)
			)
		)
		
		-- We remove hand skinbone from skin group and rename it
		--handBone.name = charName + "_" + side + "_bone_Hand"
		
		--gc()
		return fingerControllers
	),


	function rigToes charName side:"R" boneSize:1 =
	(
		-- Primary check, if there are no toe locators, we exit
		local locatorPrefix = "Locator_Toe_" + side + "_"
		local helperList = SSR_loc.collectLocators charName (locatorPrefix + "*")
		if helperList == false then 
		(
			SSR_gen.verbose ("! Could not find and toe locators for " + side + " side.")
			return false
		)
		
		-- If there are no ankle bone or foot bone, we cannot continue	
		local footBone = undefined
		local footBones = $'*_leg_skinBone_*'
		local ankleBone = undefined
		
		if footBones.count != 0 then 
		(
			for b in footBones do
			(
				if matchPattern b.name pattern:(charName + "_" + side + "_leg_skinBone_Toe") == true then footBone = b
			)
		)
		if footBone == undefined then
		(
			SSR_gen.verbose ("! Could not find " + side + " Toe Bone, unable to create toes.")
			return false
		)
		
		if footBones.count != 0 then 
		(
			for b in footBones do
			(
				if matchPattern b.name pattern:(charName + "_" + side + "_leg_skinBone_Foot") == true then ankleBone = b
			)
		)
		if ankleBone == undefined then
		(
			SSR_gen.verbose ("! Could not find " + side + " Ankle Bone, unable to create toes.")
			return false
		)
		
		-- Create main toe controller
		local posToe = undefined
		toeLocator = execute ("$'Locator_Leg_" + side + "_Tip'")
		if toeLocator != undefined then posToe = toeLocator.pos
		else (
			SSR_gen.verbose ("! Could not find " + side + " leg tip locator, unable to create toes.")
			return false
		)
		
		local toesControl = SSR_rig.ringCTL (posToe+[0,0,boneSize*5]) (boneSize*4) alignAxis:"Y"
		toesControl.name = charName + "_" + side + "_toes_CTL"
		toesControl.parent = footBone
		setTransformLockFlags toesControl #{1..9}

		penMod_Toes = 'PEN_Attribute_Holder 2'()
		addModifier toesControl penMod_Toes
		
		
	-----------------------------------------------------------------------------
	--
	--		Create toes
	--
	-----------------------------------------------------------------------------
		
		-- Let's look for toe locators
		local toeControllers = #(toesControl)
		local toeNames = #("Big", "Index", "Middle", "Fourth", "Little")
		local foundToeHelpers = false
		
		-- Search for the first A locator on big, index, middle, fourth and little
		-- If it exists, we start creating the finger
		for h in helperList do
		(
			for n in toeNames do
			(
				if matchPattern h.name pattern:(locatorPrefix + n + "A") == true then
				(	
					foundToeHelpers = true
					local toeControl = SSR_part.rigDigit charName locatorPrefix n side toesControl footBone ankleBone "Toe" boneSize:boneSize debug:true
					append toeControllers toeControl
				)
			)
		)
		
		-- We remove foot skinbones from skin group and rename it
		if foundToeHelpers == true then
		(
			SSR_gen.verbose "Moving toe skinBones to rig layer."
			local rigLayer = layermanager.getlayerfromname (charName + " Rig")
			footBone.name = charName + "_" + side + "_leg_bone_Toe"
			rigLayer.addNode footBone
		)
		
		return toeControllers
	),
	
	function doPart charName extParams:#("R", "fingers") boneSize:1 =
	(
		local controllers = undefined

		if extParams[2] == "fingers" then
		(
			controllers = SSR_part.rigFingers charName side:extParams[1] boneSize:boneSize
			return controllers
		)
		
		if extParams[2] == "toes" then
		(
			controllers = SSR_part.rigToes charName side:extParams[1] boneSize:boneSize
			return controllers
		)
		
		if extParams == undefined then return undefined
	)
)

SSR_part = SSR_part()