-----------------------------------------------------------------------------
--
--		Rig Extras Setup
--		
--
-----------------------------------------------------------------------------

global SSR_part

struct SSR_part
(
	function doPart charName extParams:#("") boneSize:1 debug:false =
	(
		local skinBones = #()
		local controllers = #()
		
		-- Collect all extra locators in the scene
		local extraLocs = SSR_loc.collectLocators charName ("*Locator_Xtra_*")
		
		if extraLocs != false then
		(
			SSR_gen.verbose ("Creating extra rig controllers on " + charName + ".")
			if debug == true then SSR_gen.verbose ("Extra Locator count: " + extraLocs.count as string)
			
			-- Sort them by finding root names, keeping only uniques
			local locRoots = #()
			for l in extraLocs do
			(
				local strippedName = subString l.name 1 (l.name.count-1)
				appendIfUnique locRoots strippedName
			)
			
			
			-- Create bone chain
			for l in locRoots do
			(
				-- Find all locators in the chain
				SSR_gen.verbose ("Creating controller chain for " + l + " on " + charName + ".")
				local curChain = execute ("$'" + l + "*'")
				local limbName = substring l 14 (l.count)
				local prefix = charName + "_" + limbName
				
				if debug == true then SSR_gen.verbose ("Prefix: " + prefix)
				if debug == true then SSR_gen.verbose (curChain.count as string + " steps in current chain.")
				
				-- Get all chain positions
				local chainPos = #()
				for c = 1 to curChain.count do
				(
					local thisPos = execute ("$'" + l + c as string + "'.pos")
					append chainPos thisPos
				)
				
				-- Figure out the best bone zAxis. If the initial bone direction is close to the y axis, we will set
				-- the bone zAxis to along the x axis.
				local zAxis = [1,0,0]
				local boneDir = normalize ( chainPos[2] - chainPos[1] )
				local zAngle = acos (dot zAxis boneDir)
				
				if zAngle < 30 or zAngle > 150 then 
				(
					zAxis = [0,1,0]
					if debug == true then SSR_gen.verbose ("zAxis along Y.")
				)
				
				-- Create the first chain link. If there are no more steps, create endbone as well.
				local boneStep	= SSR_rig.makeBone (prefix + "_skinBone_1") chainPos[1] chainPos[2] zAxis boneSize
				local topNode = SSR_rig.makeHelper chainPos[1] (boneSize*1.2) (prefix + "TOP") box:true
				SSR_rig.objAlign topNode boneStep
				boneStep.parent = topNode
				
				-- Create animation controller for this bone
				local newCon = SSR_rig.createFKController charName boneStep limbName doCon:true

				if curChain.count == 2 then 
				(
					local boneStepEnd = SSR_rig.makeBone (prefix + "_boneEnd") chainPos[2] (chainPos[2] + [0,0,boneSize]) zAxis boneSize
					SSR_rig.objAlign boneStepEnd boneStep
					boneStepEnd.parent = boneStep
				)
				
				append skinBones boneStep
				append controllers newCon

				-- Additional chain links
				if curChain.count > 2 then
				(
					for c = 2 to (curChain.count - 1) do
					(
						if debug == true then SSR_gen.verbose ("On step: " + c as string + ".")
						local boneStep	= SSR_rig.makeBone (prefix + "_skinBone_" + c as string) chainPos[c] chainPos[c+1] zAxis boneSize
						
						-- Parent this bone to the previous one
						execute ( "$'" + prefix + "_skinBone_" + c as string + "'.parent = $'"  + prefix + "_skinBone_" + (c-1) as string + "'")
						
						-- Create animation controller for this bone
						SSR_rig.nullTransform boneStep
						local newCon = SSR_rig.createFKController charName boneStep (limbName + (c-1) as string) doCon:false
						
						if c == (curChain.count - 1) do
						(
							local boneStepEnd = SSR_rig.makeBone (prefix + "_boneEnd") chainPos[c+1] (chainPos[c+1] + [0,0,boneSize]) zAxis boneSize
							SSR_rig.objAlign boneStepEnd boneStep
							boneStepEnd.parent = boneStep		
						)
						
						append skinBones boneStep
						append controllers newCon
					)
				) -- if curChain.count > 2 then
			) -- for l in locRoots do

			
			return controllers
		) else (
			SSR_gen.verbose ("No extra rig helpers found on character " + charName + ".")
			return true
		)
	)
) 

SSR_part = SSR_part()