-----------------------------------------------------------------------------
--
--		Super Simple Rig
--		Standard head setup
--
-----------------------------------------------------------------------------
global SSR_part

struct SSR_part
(
	function doPart charName boneSize:1 =
	(
		local helperList = SSR_loc.collectLocators charName ("*Locator_Head_*")
		
		if helperList == false then return false
		
		local posNeck, posMidNeck, posHead, posEyeR, posEyeL, posJaw, posInMouth, posMouth, posHeadTop
		local prefix = (charName + "_head_")
		
		local ctlColor = (color 200 200 0)	
		
		-- Look for helper objects that give locations to spine bone positions
		if helperList != false then for h in helperList do
		(
			if matchPattern h.name pattern:("*_Head_Neck") 		== true then posNeck = h.pos
			if matchPattern h.name pattern:("*_Head_MidNeck") 	== true then posMidNeck = h.pos
			if matchPattern h.name pattern:("*_Head_Head") 		== true then posHead = h.pos
			if matchPattern h.name pattern:("*_Head_R_Eye") 	== true then posEyeR = h.pos
			if matchPattern h.name pattern:("*_Head_L_Eye") 	== true then posEyeL = h.pos
			if matchPattern h.name pattern:("*_Head_Jaw") 		== true then posJaw = h.pos
			if matchPattern h.name pattern:("*_Head_InMouth") 	== true then posInMouth = h.pos
			if matchPattern h.name pattern:("*_Head_Mouth") 	== true then posMouth = h.pos
			if matchPattern h.name pattern:("*_Head_HeadTop") 	== true then posHeadTop = h.pos
		)
		
		-- If one of these helpers are not found, we will exit
		if posNeck == undefined or posMidNeck == undefined or posHead == undefined \
			or posEyeR == undefined or posEyeL == undefined or posJaw == undefined \
			or posMouth == undefined or posInMouth == undefined or posHeadTop == undefined 
		then (
			SSR_gen.verbose ("Missing locators for head, skipping head rig.")
			return false
		) else (
			SSR_gen.verbose "All head locators found, creating head rig."
		)
		
	-----------------------------------------------------------------------------
	--
	--		Bone objects
	--
	-----------------------------------------------------------------------------	
		
		--local boneSize = (distance posNeck posHeadTop) / 20
		local zAxis = [-1, 0, 0]
		
		-- Neck and head bones
		local boneNeckLow 	= SSR_rig.makeBone (prefix + "skinBone_LowNeck") posNeck posMidNeck zAxis boneSize
		local boneNeckHigh	= SSR_rig.makeBone (prefix + "skinBone_HighNeck") posMidNeck posHead zAxis boneSize
		local boneHead		= SSR_rig.makeBone (prefix + "skinBone_Head") posHead posHeadTop zAxis boneSize
		local boneHeadEnd	= SSR_rig.makeBone (prefix + "bone_HeadEnd") posHeadTop (posHeadTop + [0, 0, boneSize]) zAxis boneSize
		SSR_rig.objAlign boneHeadEnd boneHead
		
		-- Jaw and mouth bones
		local boneJaw		= SSR_rig.makeBone (prefix + "skinBone_Jaw") posJaw posInMouth zAxis boneSize
		local boneMouth		= SSR_rig.makeBone (prefix + "skinBone_Mouth") posInMouth posMouth zAxis boneSize
		local boneMouthEnd	= SSR_rig.makeBone (prefix + "bone_MouthEnd") posMouth (posMouth + [0, -boneSize, 0]) zAxis boneSize
		SSR_rig.objAlign boneMouthEnd boneMouth	
		
		-- Eye bones
		local boneEyeR		= SSR_rig.makeBone (prefix + "skinBone_REye") posEyeR (posEyeR+[0,-boneSize*2,0]) zAxis boneSize
		local boneEyeL		= SSR_rig.makeBone (prefix + "skinBone_LEye") posEyeL (posEyeL+[0,-boneSize*2,0]) zAxis boneSize
		
		-- Parenting
		boneNeckHigh.parent = boneNeckLow
		boneHead.parent = boneNeckHigh
		boneHeadEnd.parent = boneHead
		
		boneJaw.parent = boneHead
		boneMouth.parent = boneJaw
		boneMouthEnd.parent = boneMouth
		
		local boneEyeREnd	= SSR_rig.endBone boneEyeR
		boneEyeREnd.name = (prefix + "bone_REyeEnd")
		local boneEyeLEnd	= SSR_rig.endBone boneEyeL
		boneEyeLEnd.name = (prefix + "bone_LEyeEnd")
		
		-- Null transforms and helpers
		local headRoot = SSR_rig.nullTransform boneNeckLow
		headRoot.name = prefix + "Root"
		local headLocal = SSR_rig.nullTransform boneHead
		headLocal.name = prefix + "headLocal"
		
		local mouthRoot = SSR_rig.nullTransform boneMouth
		mouthRoot.name = prefix + "mouthBase"
		
		local jawRoot = SSR_rig.makeHelper posJaw boneSize (prefix + "JawBase") box:true
		SSR_rig.objAlign jawRoot boneJaw
		boneJaw.parent = jawRoot
		jawRoot.parent = boneHead
		
		-- Create camera for facial animation
		local faceCam = freeCamera pos:[0, (posMouth.y - boneSize*20 ), ((posEyeR.z + posMouth.z) * 0.5)]
		rotate faceCam (angleaxis 90 [1,0,0])
		faceCam.parent = boneHead
		faceCam.name = charName + "_faceCam"
		--faceCam.isHidden = true
		
	-----------------------------------------------------------------------------
	--
	--		Head Control object
	--
	-----------------------------------------------------------------------------	
		
		local neckControl = SSR_rig.ringCTL posNeck (boneSize*15) alignAxis:"X"
		neckControl.name = (charName + "_neck_CTL")
		SSR_rig.objAlign neckControl boneNeckLow
		setTransformLockFlags neckControl #{1..3,7..9}
		
		local headControl = SSR_rig.pinCTL posHead (boneSize*10) (boneSize*5)
		headControl.name = (prefix + "CTL")
		headControl.parent = headLocal
		SSR_rig.objAlign headControl boneHead
		boneHead.parent = headControl
		SSR_rig.nullTransform headControl
		setTransformLockFlags headControl #{7..9}
		
		-- The head and neck bones will be constrained gradually from the lower neck to the upper neck
		local highNeckCon = SSR_rig.rotConTwo boneNeckHigh neckControl headRoot
		highNeckCon.relative = on
		highNeckCon.weight[1] = 100.0
		highNeckCon.weight[2] = 0
			
		local lowNeckCon = SSR_rig.rotConTwo boneNeckLow neckControl headRoot
		lowNeckCon.relative = on
		lowNeckCon.weight[1] = 50.0
		lowNeckCon.weight[2] = 50.0
		
		-- This helper will make the head switch between world and local orientation
		local headAlign = SSR_rig.makeHelper posNeck (boneSize*2) (prefix + "headConstraint_Align") cross:true
		SSR_rig.objAlign headAlign neckControl
		neckControl.parent = headAlign
		
		local neckRoot = SSR_rig.makeHelper posNeck (boneSize*2) (prefix + "headConstraint_Root") cross:true
		headAlign.parent = neckRoot
		
		-- Jaw controller
		local jawControl = SSR_rig.pinCTL posJaw ((distance posJaw posMouth)*1.5) (boneSize*3) alignAxis:"X"
		SSR_rig.objAlign jawControl boneJaw
		jawControl.name = (charName + "_jaw_CTL")
		setTransformLockFlags jawControl #{7..9}
		local jawNull = SSR_rig.nullTransform jawControl
		boneJaw.parent = jawControl
		jawControl.parent = jawRoot
		
		-- Face and eyes controllers to put attributes for morph targets and such
		local eyesControl = text pos:(posHead + [-boneSize*30,0,boneSize*2]) size:(boneSize*10) text:":" steps:1
		eyesControl.name = charName + "_eyes_CTL"
		
		eyesControl.pivot = eyesControl.center
		in coordsys local rotate eyesControl (eulerAngles 90 180 0) 
		ResetXForm eyesControl
		convertTo eyesControl SplineShape
		
		eyesControl.parent = headControl

		local faceControl = text pos:(posHead + [-boneSize*30,0,0]) size:(boneSize*10) text:")" steps:1
		faceControl.name = charName + "_face_CTL"
		
		faceControl.pivot = faceControl.center
		in coordsys local rotate faceControl (eulerAngles 90 180 0) 
		ResetXForm faceControl
		convertTo faceControl SplineShape
		
		faceControl.parent = headControl
		
	-----------------------------------------------------------------------------
	--
	--		Head constraints
	--
	-----------------------------------------------------------------------------

		-- We set up constraints to slide the head from world to local (spine) rotation
		local headWorldHelper = SSR_rig.makeHelper neckControl.pos (boneSize*2) (prefix + "headConstraint_World") box:true
		local headLocalHelper = SSR_rig.makeHelper neckControl.pos (boneSize*1.5) (prefix + "headConstraint_Local") box:true
		
		SSR_rig.objAlign headWorldHelper neckControl
		SSR_rig.objAlign headLocalHelper neckControl
		
		headRotCon = SSR_rig.rotConTwo headAlign headLocalHelper headWorldHelper
		headLocalHelper.parent = headRoot
		
	-----------------------------------------------------------------------------
	--
	--		Eye Control objects and lookat constraints
	--
	-----------------------------------------------------------------------------
		
		local lookControl = SSR_rig.crossCTL [0.0,( posEyeR.y - ((distance posNeck posHeadTop)*4) ), posEyeR.z] (boneSize*5)
		lookControl.name = charName + "_lookat_CTL"
		setTransformLockFlags lookControl #{4..9}
		
		local eyeRControl = SSR_rig.ringCTL [posEyeR.x, lookControl.pos.y, posEyeR.z] (boneSize) alignAxis:"Y"
		eyeRControl.name = charName + "_eye_posR_CTL"
		eyeRControl.parent = lookControl
		setTransformLockFlags eyeRControl #{2,4..9}
		
		local eyeLControl = SSR_rig.ringCTL [posEyeL.x, lookControl.pos.y, posEyeL.z] (boneSize) alignAxis:"Y"
		eyeLControl.name = charName + "_eye_posL_CTL"
		eyeLControl.parent = lookControl
		setTransformLockFlags eyeLControl #{2,4..9}
		
		local eyeRParent = SSR_rig.makeHelper posEyeR (boneSize*3) (prefix + "eyeR_Parent") box:true
		local eyeLParent = SSR_rig.makeHelper posEyeL (boneSize*3) (prefix + "eyeL_Parent") box:true
		
		local eyeRNull = SSR_rig.nullTransform eyeRParent
		local eyeLNull = SSR_rig.nullTransform eyeLParent
		
		eyeRNull.parent = eyeLNull.parent = boneHead
		
		local RCon = SSR_rig.lookCon eyeRParent eyeRControl
		RCon.StoUp_axis = 1
		RCon.StoUP_axisFlip = false
		RCon.target_axis = 2
		RCon.target_axisFlip = off
		RCon.upnode_axis = 2
		RCon.pickUpNode = eyeRNull
		RCon.upnode_world = off
		
		local LCon = SSR_rig.lookCon eyeLParent eyeLControl
		RCon.StoUp_axis = 1
		RCon.StoUP_axisFlip = false
		LCon.target_axis = 2
		LCon.target_axisFlip = off
		LCon.upnode_axis = 2
		LCon.pickUpNode = eyeLNull
		LCon.upnode_world = off
		
	-----------------------------------------------------------------------------
	--
	--		Eye local/world orientation and position constraints
	--
	-----------------------------------------------------------------------------
		
		local eyeWorldParent = SSR_rig.makeHelper posHead (boneSize*2) (prefix + "eyeConstraint_WorldParent") box:true
		local eyeLocalParent = SSR_rig.makeHelper posHead (boneSize*1.5) (prefix + "eyeConstraint_LocalParent")	box:true
		
		local eyeWorldHelper = SSR_rig.makeHelper lookControl.pos (boneSize*2) (prefix + "eyeConstraint_World") box:true
		local eyeLocalhelper = SSR_rig.makeHelper lookControl.pos (boneSize*1.5) (prefix + "eyeConstraint_Local") box:true

		eyeWorldHelper.parent = eyeWorldParent
		eyeLocalHelper.parent = eyeLocalParent
		eyeLocalParent.parent = headControl
		
		global eyeLookCon = SSR_rig.lookCon lookControl headControl
		eyeLookCon.StoUp_axis = 2
		eyeLookCon.StoUP_axisFlip = false
		eyeLookCon.relative = on
		eyeLookCon.upnode_world = off
		eyeLookCon.pickUpNode = boneHead
		eyeLookCon.upNode_axis = 0
		
		local eyePosCon = SSR_rig.posConTwo lookControl eyeWorldHelper eyeLocalhelper
		eyePosCon.weight[1] = 0
		eyePosCon.weight[2] = 100
		
		local eyeListCon = Position_List()
		lookControl.pos.controller = eyeListCon
		local eyeEuler = Position_XYZ()
		eyeListCon.available.controller = eyeEuler
		eyeListCon.active = 2
		
	-----------------------------------------------------------------------------
	--
	--		An additional eye override system using rotate controllers
	--
	-----------------------------------------------------------------------------	
		
		local eyeRotParentR = SSR_rig.makeHelper posEyeR (boneSize*2) (prefix + "eyeR_Parent") box:true
		local eyeRotParentL = SSR_rig.makeHelper posEyeL (boneSize*2) (prefix + "eyeL_Parent") box:true
		
		local eyeRotControlR = SSR_rig.ballCTL (posEyeR + [0, -boneSize * 5, 0]) boneSize
		local eyeRotControlL = SSR_rig.ballCTL (posEyeL + [0, -boneSize * 5, 0]) boneSize
		eyeRotControlR.name = charName + "_eye_R_CTL"
		eyeRotControlL.name = charName + "_eye_L_CTL"
		
		setTransformLockFlags eyeRotControlR #{1..3,7..9}
		setTransformLockFlags eyeRotControlL #{1..3,7..9}
		
		eyeRotControlR.parent = eyeRParent
		eyeRotControlL.parent = eyeLParent
		eyeRotParentR.parent = eyeRParent
		eyeRotParentL.parent = eyeLParent
		BoneEyeR.parent = eyeRotParentR
		BoneEyeL.parent = eyeRotParentL
		
		SSR_rig.rotCon eyeRotParentR eyeRotControlR 
		SSR_rig.rotCon eyeRotParentL eyeRotControlL
		
		SSR_rig.NullTransform eyeRotControlR
		SSR_rig.NullTransform eyeRotControlL
		
	-----------------------------------------------------------------------------
	--
	--		Custom attributes for the neck controller
	--
	-----------------------------------------------------------------------------	
		
		local neckTranAtt = attributes transforms
		(
			parameters params rollout:neckRoll
			(
				X_Rot		type:#angle ui:RXSp
				Y_Rot		type:#angle ui:RYSp
				Z_Rot		type:#angle ui:RZSp
			)
			rollout neckRoll "Transforms"
			(
				spinner RXSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner RYSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner RZSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)		
		
		-- Add custom attributes to the neck controller
		local neckMod = 'PEN_Attribute_Holder 2'()
		addModifier neckControl neckMod
		custAttributes.add neckMod neckTranAtt
		
		-- Connect up custom attributes for the neck
		neckMod.transforms.X_Rot.controller = \
		neckControl.rotation.controller.X_Rotation.controller
		neckMod.transforms.Y_Rot.controller = \
		neckControl.rotation.controller.Y_Rotation.controller
		neckMod.transforms.Z_Rot.controller = \
		neckControl.rotation.controller.Z_Rotation.controller

	-----------------------------------------------------------------------------
	--
	--		Custom attributes for the head controller
	--
	-----------------------------------------------------------------------------	

		local headTranAtt = attributes transforms
		(
			parameters params rollout:neckRoll
			(
				X_Pos		type:#float ui:PXSp
				Y_Pos		type:#float ui:PYSp
				Z_Pos		type:#float ui:PZSp
				X_Rot		type:#angle ui:RXSp
				Y_Rot		type:#angle ui:RYSp
				Z_Rot		type:#angle ui:RZSp
			)
			rollout neckRoll "Head Transforms"
			(
				spinner PXSp 	"Translate X " 	range:[-1000000, 1000000, 0]
				spinner PYSp 	"Translate Y " 	range:[-1000000, 1000000, 0]
				spinner PZSp 	"Translate Z " 	range:[-1000000, 1000000, 0]
				spinner RXSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner RYSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner RZSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)
		local headConAtt = attributes constraints
		(
			parameters params rollout:headRoll
			(
				headPos_WL	type:#float ui:(headSp, headSl)
			)
			rollout headRoll "Constraints"
			(
				label lbl01 		"Head orients Spine <> World" align:#centre
				spinner headSp 		"" range:[0, 100, 0] fieldwidth:40 align:#left offset:[10, 10] across:2
				slider headSl 		"" range:[0, 100, 0]
			)
		)
		
		-- Add custom attributes to the head controller
		local headMod = 'PEN_Attribute_Holder 2'()
		addModifier headControl headMod
		custAttributes.add headMod headTranAtt
		custAttributes.add headMod headConAtt
		
		-- Connect up custom attributes for the head
		headMod.transforms.X_Pos.controller =\
		headControl.pos.controller.X_Position.controller
		headMod.transforms.Y_Pos.controller =\
		headControl.pos.controller.Y_Position.controller
		headMod.transforms.Z_Pos.controller =\
		headControl.pos.controller.Z_Position.controller
		
		headMod.transforms.X_Rot.controller =\
		headControl.rotation.controller.X_Rotation.controller
		headMod.transforms.Y_Rot.controller =\
		headControl.rotation.controller.Y_Rotation.controller
		headMod.transforms.Z_Rot.controller =\
		headControl.rotation.controller.Z_Rotation.controller

		paramWire.connect headMod.constraints[#headPos_WL] \
			headAlign.rotation.controller[#Orientation_Weight_0] \
			"100-headPos_WL"
		paramWire.connect headMod.constraints[#headPos_WL] \
			headAlign.rotation.controller[#Orientation_Weight_1] \
			"headPos_WL"

	-----------------------------------------------------------------------------
	--
	--		Custom attributes for the jaw and face controllers
	--
	-----------------------------------------------------------------------------

		-- Attributes for the Jaw
		local jawAtt = attributes transforms
		(
			parameters params rollout:neckRoll
			(
				X_Pos	type:#float ui:PXSp
				Y_Pos	type:#float ui:PYSp
				Z_Pos	type:#float ui:PZSp
				X_Rot	type:#angle ui:RXSp
				Y_Rot	type:#angle ui:RYSp
				Z_Rot	type:#angle ui:RZSp
			)
			rollout neckRoll "Transforms"
			(
				spinner PXSp 	"Jaw In Out " 	range:[-1000000, 1000000, 0]
				spinner PYSp 	"Jaw Up Down " 	range:[-1000000, 1000000, 0]
				spinner PZSp 	"Jaw Left Right " 	range:[-1000000, 1000000, 0]
				spinner RXSp 	"Jaw Twist " 	range:[-1000000, 1000000, 0] align:#right
				spinner RYSp 	"Jaw Sides " 	range:[-1000000, 1000000, 0] align:#right
				spinner RZSp 	"Jaw Open Close " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	
		
		-- Add custom attributes to the face and eye controller
		local faceMod = 'PEN_Attribute_Holder 2'()
		addModifier faceControl faceMod
		custAttributes.add faceMod jawAtt
		
		local eyesMod = 'PEN_Attribute_Holder 2'()
		addModifier eyesControl eyesMod
		
		-- Add custom attributes to the jaw controller
		local jawMod = 'PEN_Attribute_Holder 2'()
		addModifier jawControl jawMod
		custAttributes.add jawMod jawAtt
		
		jawMod.transforms[#X_Pos].controller = faceMod.transforms[#X_Pos].controller = \
		jawControl.pos.controller.X_Position.controller
		jawMod.transforms[#Y_Pos].controller = faceMod.transforms[#Y_Pos].controller = \
		jawControl.pos.controller.Y_Position.controller
		jawMod.transforms[#Z_Pos].controller = faceMod.transforms[#Z_Pos].controller = \
		jawControl.pos.controller.Z_Position.controller
		
		jawMod.transforms[#X_Rot].controller = faceMod.transforms[#X_Rot].controller = \
		jawControl.rotation.controller.X_Rotation.controller
		jawMod.transforms[#Y_Rot].controller = faceMod.transforms[#Y_Rot].controller = \
		jawControl.rotation.controller.Y_Rotation.controller
		jawMod.transforms[#Z_Rot].controller = faceMod.transforms[#Z_Rot].controller = \
		jawControl.rotation.controller.Z_Rotation.controller

		-- Attributes for the outer Mouth bone
		local mouthAtt = attributes mouth_transforms
		(
			parameters params rollout:mouthTrans
			(
				X_Pos		type:#float ui:X_PosSp
				Y_Pos		type:#float ui:Y_PosSp
				Z_Pos		type:#float ui:Z_PosSp
				X_Rot		type:#angle ui:X_RotSp
				Y_Rot		type:#angle ui:Y_RotSp
				Z_Rot		type:#angle ui:Z_RotSp
			)
			rollout mouthTrans "Mouth Transforms"
			(
				spinner X_RotSp 	"Mouth Twist " 		range:[-1000000, 1000000, 0] align:#right
				spinner Y_RotSp 	"Mouth Left Right " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_RotSp 	"Mouth Up Down " 	range:[-1000000, 1000000, 0] align:#right
				spinner X_PosSp 	"Mouth In Out " 		range:[-1000000, 1000000, 0] align:#right offset:[0,10]
				spinner Y_PosSp 	"Mouth Up Down " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_PosSp 	"Mouth Left Right " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	
		custAttributes.add faceMod mouthAtt

		-- Connect up the mouth attributes
		faceMod.mouth_transforms[#X_Pos].controller = \
		boneMouth.pos.controller.X_Position.controller
		faceMod.mouth_transforms[#Y_Pos].controller = \
		boneMouth.pos.controller.Y_Position.controller
		faceMod.mouth_transforms[#Z_Pos].controller = \
		boneMouth.pos.controller.Z_Position.controller
		
		paramWire.connect2Way faceMod.mouth_transforms[#X_Rot] \
			boneMouth.rotation.controller[#X_Rotation] \
			"radToDeg X_Rotation" "degToRad X_Rot"
		paramWire.connect2Way faceMod.mouth_transforms[#Y_Rot] \
			boneMouth.rotation.controller[#Y_Rotation] \
			"radToDeg Y_Rotation" "degToRad Y_Rot"
		paramWire.connect2Way faceMod.mouth_transforms[#Z_Rot] \
			boneMouth.rotation.controller[#Z_Rotation] \
			"radToDeg Z_Rotation" "degToRad Z_Rot"

	-----------------------------------------------------------------------------
	--
	--		Custom attributes for the eye controller
	--
	-----------------------------------------------------------------------------

		-- Add custom attributes to the eye controller
		local eyeMod = 'PEN_Attribute_Holder 2'()
		addModifier lookControl eyeMod
		
		local eyeTranAtt = attributes transforms
		(
			parameters params rollout:eyeRoll
			(
				X_Pos		type:#float ui:PXSp
				Y_Pos		type:#float ui:PYSp
				Z_Pos		type:#float ui:PZSp
				Eyes_Follow	type:#float ui:(neckSp, neckSl)
			)
			rollout eyeRoll "Transforms"
			(
				spinner PXSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner PYSp 	"Translate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner PZSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
				label lbl01 		"Eyes follows World <> Head" align:#centre offset:[0, 10]
				spinner neckSp 		"" range:[0, 100, 0] fieldwidth:40 align:#left offset:[10, 10] across:2
				slider neckSl 		"" range:[0, 100, 0]
			)
		)	
		custAttributes.add eyeMod eyeTranAtt

		eyeMod.transforms[#X_Pos].controller = \
		lookControl.pos.controller.Position_XYZ.controller.X_Position.controller
		eyeMod.transforms[#Y_Pos].controller = \
		lookControl.pos.controller.Position_XYZ.controller.Y_Position.controller
		eyeMod.transforms[#Z_Pos].controller = \
		lookControl.pos.controller.Position_XYZ.controller.Z_Position.controller
		
		paramWire.connect eyeMod.transforms[#Eyes_Follow] \
			lookControl.pos.controller.Position_Constraint.controller[#Position_Weight_0] \
			"100-Eyes_Follow"
		paramWire.connect eyeMod.transforms[#Eyes_Follow] \
			lookControl.pos.controller.Position_Constraint.controller[#Position_Weight_1] \
			"Eyes_Follow"
		
		local eyeRAtt = attributes r_eye_transforms
		(
			parameters params rollout:eyeRoll
			(
				X_Pos		type:#float ui:PXSp
				Z_Pos		type:#float ui:PZSp
			)
			rollout eyeRoll "Right Eye Transforms"
			(
				spinner PXSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner PZSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	
		custAttributes.add eyeMod eyeRAtt	
		
		SSR_rig.NullTransform eyeRControl
		
		eyeMod.r_eye_transforms[#X_Pos].controller = \
		eyeRControl.pos.controller.X_Position.controller
		eyeMod.r_eye_transforms[#Z_Pos].controller = \
		eyeRControl.pos.controller.Z_Position.controller
		
		local eyeLAtt = attributes l_eye_transforms
		(
			parameters params rollout:eyeRoll
			(
				X_Pos		type:#float ui:PXSp
				Z_Pos		type:#float ui:PZSp
			)
			rollout eyeRoll "Left Eye Transforms"
			(
				spinner PXSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner PZSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	
		custAttributes.add eyeMod eyeLAtt		

		SSR_rig.NullTransform eyeLControl
		
		eyeMod.l_eye_transforms[#X_Pos].controller = \
		eyeLControl.pos.controller.X_Position.controller
		eyeMod.l_eye_transforms[#Z_Pos].controller = \
		eyeLControl.pos.controller.Z_Position.controller
		
	-----------------------------------------------------------------------------
	--
	--		Custom attributes for the eye rotate controllers
	--
	-----------------------------------------------------------------------------	
		
		-- Custom attributes for the right eye rotate controller
		local eyeRotAtt = attributes eye_transforms
		(
			parameters params rollout:roll_eyeR
			(
				X_Rot		type:#angle ui:RXSp
				Y_Rot		type:#angle ui:RYSp
				Z_Rot		type:#angle ui:RZSp
			)
			rollout roll_eyeR "Eye Rotations"
			(
				spinner RXSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner RYSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner RZSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	
		
		local eyeRMod = 'PEN_Attribute_Holder 2'()
		addModifier eyeRotControlR eyeRMod
		custAttributes.add eyeRMod eyeRotAtt

		local eyeLMod = 'PEN_Attribute_Holder 2'()
		addModifier eyeRotControlL eyeLMod
		custAttributes.add eyeLMod eyeRotAtt
		
		eyeRMod.eye_transforms[#X_Rot].controller = \
		eyeRotControlR.rotation.controller.X_Rotation.controller
		eyeRMod.eye_transforms[#Y_Rot].controller = \
		eyeRotControlR.rotation.controller.Y_Rotation.controller
		eyeRMod.eye_transforms[#Z_Rot].controller = \
		eyeRotControlR.rotation.controller.Z_Rotation.controller
		
		eyeLMod.eye_transforms[#X_Rot].controller = \
		eyeRotControlL.rotation.controller.X_Rotation.controller
		eyeLMod.eye_transforms[#Y_Rot].controller = \
		eyeRotControlL.rotation.controller.Y_Rotation.controller
		eyeLMod.eye_transforms[#Z_Rot].controller = \
		eyeRotControlL.rotation.controller.Z_Rotation.controller
		
		
		return headControllers
	)
)

SSR_part = SSR_part()