-----------------------------------------------------------------------------
--
--		Super Character Rig
--		FK IK Leg Setup
--
-----------------------------------------------------------------------------
global SSR_part

struct SSR_part
(
	function doPart charName extParams:#("R", true, 3, 3) boneSize:1 debug:false =
	(
		-- External parameters
		local side = extParams[1] 
		local doSoft = 1 -- Should be optional, slows down the rig?
		local doTwist = extParams[2]
		local numTwistShin = extParams[3]
		local numTwistThigh = extParams[4]
		
		local helperList = SSR_loc.collectLocators charName ("*Locator_Leg_" + side +"_*")
		local posHip, posKnee, posAnkle, posHeel, posBall, posToe, posSideOut, posSideIn = undefined
		local LegRoot
		local prefix = (charName + "_" + side + "_leg_")

		-- Look for helper objects that give locations to leg bone positions
		if helperList != false then for h in helperList do
		(
			if matchPattern h.name pattern:("*_Leg_" + side + "_Hip*") 		== true then posHip = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_Knee*") 	== true then posKnee = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_Ankle*") 	== true then posAnkle = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_Heel*") 	== true then posHeel = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_Toe*") 		== true then posBall = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_Tip*") 		== true then posToe = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_SideOut*") 	== true then posSideOut = h.pos
			if matchPattern h.name pattern:("*_Leg_" + side + "_SideIn*") 	== true then posSideIn = h.pos
		)
		
		-- If one of these helpers are not found, we will exit
		if posHip == undefined or posKnee == undefined or posAnkle == undefined \
			or posHeel == undefined or posBall == undefined or posToe == undefined \
			or posSideIn == undefined or posSideOut == undefined
		then (
			SSR_gen.verbose ("Missing locators for " + side as string + " leg.")
			return false
		) else (
			SSR_gen.verbose ("All locators are found, now creating " + side as string + " leg.")
		)

		
	-----------------------------------------------------------------------------
	--
	--		Create IK and FK system
	--
	-----------------------------------------------------------------------------
		
		-- Bone zAxis for the leg will be perpendicular to the bone plane
		local legAxis = normalize ( cross (posHip-posKnee) (posAnkle-posKnee) )
		-- Bone zAxis for the foot will be along the ground
		local footAxis = normalize (cross (posToe-posHeel) [0,0,10])
		
		-- returns #( #(boneStart, boneEnd),#(IK_root, IK_target, midMover), limit, #(midLookat, midStartEnd, midNull) )
		local IKleg = SSR_rig.IKSetup prefix "Thigh" "Knee" "Shin" posHip posKnee posAnkle boneSize:boneSize debug:debug 
		
		local kneePinControl 	= IKleg[1][1]
		local kneeLookatControl = IKleg[1][2]
		local boneThigh_IK 	= IKleg[2][1]
		local boneShin_IK 	= IKleg[2][2]
		local IKroot 		= IKleg[3][1]
		local IKend 		= IKleg[3][2]
		local kneePin 		= IKleg[3][3]
		local limitCon 		= IKleg[4]
		local kneeLookat 	= IKleg[5][1]
		local kneeHipFoot 	= IKleg[5][2]
		local kneeWorldNull = IKleg[5][3]
		
		-- Create the IK bones, thigh and shin are replaced with IKleg function above
		kneePinControl.name 	= charName + "_" + side + "_kneePin_IK_CTL"
		kneeLookatControl.name 	= charName + "_" + side + "_knee_IK_CTL"
		boneThigh_IK.name 		= (prefix + "IKBone_Thigh")
		boneShin_IK.name 		= (prefix + "IKBone_Shin")
		
		--! seems like the shin bone need a different upnode alignment when created individually. 
		--! Beware, figure out why
		--boneShin_IK.rotation.controller.StoUP_axisFlip = off
		
		-- The FK arm system
		local flipSide = false
		if side == "L" then flipSide = true
		local FKleg = SSR_rig.FKSetup prefix "Thigh" "Knee" "Shin" posHip posKnee posAnkle boneSize:boneSize zAxis:zAxis flipSide:flipSide debug:debug 
		
		local hipControl 	= FKleg[1][1]
		local kneeFKControl = FKleg[1][2]
		local boneThigh_FK 	= FKleg[2][1]
		local boneShin_FK 	= FKleg[2][2]
		
		-- Rename the FK objects
		hipControl.name = charName + "_" + side + "_hip_CTL"
		kneeFKControl.name = charName + "_" + side + "_knee_CTL"
		boneThigh_FK.name 	= (prefix + "FKBone_Thigh")
		boneShin_FK.name	= (prefix + "FKBone_Shin")

	-----------------------------------------------------------------------------
	--
	--		Create and link FK, IK and skinning bones
	--
	-----------------------------------------------------------------------------

		
		-- Create our main skinning bones
		local boneThigh 		= SSR_rig.makeBone (prefix + "skinBone_Thigh") posHip posKnee legAxis boneSize
		local boneShin 			= SSR_rig.makeBone (prefix + "skinBone_Shin") posKnee posAnkle legAxis boneSize
		local boneFoot			= SSR_rig.makeBone (prefix + "skinBone_Foot") posAnkle posBall legAxis boneSize
		local boneToe			= SSR_rig.makeBone (prefix + "skinBone_Toe") posBall posToe footAxis boneSize
		local boneToeEnd		= SSR_rig.makeBone (prefix + "bone_ToeEnd") posToe (posToe + [0,0,boneSize]) footAxis boneSize

		-- IK bones for the foot
		local boneFoot_IK 		= SSR_rig.makeBone (prefix + "IKBone_Foot") posHeel posToe footAxis boneSize
		local boneBall_IK 		= SSR_rig.makeBone (prefix + "IKBone_Ball") posToe posBall footAxis boneSize
		local boneAnkle_IK 		= SSR_rig.makeBone (prefix + "IKBone_Ankle") posBall posAnkle footAxis boneSize
		local boneAnkleEnd_IK	= SSR_rig.makeBone (prefix + "IKBone_AnkleEnd") posAnkle (posAnkle + [0,0,-boneSize]) footAxis boneSize
		local boneToe_IK 		= SSR_rig.makeBone (prefix + "IKBone_Toe") posBall posToe footAxis boneSize
		local boneToeEnd_IK		= SSR_rig.makeBone (prefix + "IKBone_ToeEnd") posToe (posToe + [0,0,boneSize]) footAxis boneSize
		
		-- FK bones for the foot
		local boneFoot_FK		= SSR_rig.makeBone (prefix + "FKBone_Foot") posAnkle posBall footAxis boneSize
		local boneToe_FK		= SSR_rig.makeBone (prefix + "FKBone_Toe") posBall posToe footAxis boneSize
		local boneToeEnd_FK		= SSR_rig.makeBone (prefix + "FKBone_ToeEnd") posToe (posToe + [0,0,boneSize]) footAxis boneSize
		
		-- IK foot parenting
		SSR_rig.objAlign boneAnkleEnd_IK boneAnkle_IK
		SSR_rig.objAlign boneToeEnd_IK boneToe_IK
		boneAnkleEnd_IK.parent = boneAnkle_IK
		boneAnkle_IK.parent = boneBall_IK
		boneBall_IK.parent = boneFoot_IK
		boneToeEnd_IK.parent = boneToe_IK
		boneToe_IK.parent = boneBall_IK
		
		-- IK Bones will be bright red
		boneFoot_IK.wireColor = boneBall_IK.wireColor = boneAnkle_IK.wireColor = \
			boneAnkleEnd_IK.wireColor = boneToe_IK.wireColor = boneToeEnd_IK.wireColor = \
			boneThigh_IK.wireColor = boneShin_IK.wireColor = [255,128,128] --boneShinEnd_IK.wireColor = 
		
		-- skinBones alignments and parenting
		SSR_rig.objAlign boneToeEnd boneToe
		boneToeEnd.parent = boneToe
		boneToe.parent = boneFoot
		boneFoot.parent = boneShin
		boneShin.parent = boneThigh
		
		-- FK alignments and parenting
		SSR_rig.objAlign boneToeEnd_FK boneToe_FK
		boneToeEnd_FK.parent = boneToe_FK
		boneToe_FK.parent = boneFoot_FK
		boneFoot_FK.parent = boneShin_FK
		--boneShin_FK.parent = boneThigh_FK	
		
		-- FK Bones will be bright green
		boneThigh_FK.wireColor = boneShin_FK.wireColor = boneFoot_FK.wireColor = \
			boneToe_FK.wireColor = boneToeEnd_FK.wireColor = [128,255,128]
		
	-----------------------------------------------------------------------------
	--
	--		IK Controller objects
	--
	-----------------------------------------------------------------------------
		
		-- Foot IK controller
		-- A spline rectangle that fits around the foot locators
		local sideMult = undefined
		if side == "R" then sideMult = 1 else sideMult = -1

		local v1 = [posToe.x, posToe.y, 0] - [posHeel.x, posHeel.y, 0]  
		local theAngle = sideMult * acos (dot (normalize v1) [0,-1,0])
		if debug == true then SSR_gen.verbose ("Angle between world and leg is: " + theAngle as string)
		
		local controlLength = (distance posHeel posToe) * 1.3 
		local controlWidth = (distance posSideOut posSideIn) * 1.3

		local footIKControl = Rectangle length:controlLength width:controlWidth 
		footIKControl.rotation = (eulerAngles 0 0 theAngle)
		footIKControl.position = (posHeel + posToe) * 0.5
		
		footIKControl.pivot = posAnkle
		footIKControl.name = charName + "_" + side + "_foot_IK_CTL"
		setTransformLockFlags footIKControl #{7..9}

		
	-----------------------------------------------------------------------------
	--
	--		FK Controller objects
	--
	-----------------------------------------------------------------------------
		
		-- Foot FK controller
		-- The L side will be flipped for symmetric posing.
		local footFKControl = SSR_rig.ringCTL posAnkle (boneSize*8) alignAxis:"X" 
		footFKControl.name = charName + "_" + side + "_foot_FK_CTL"
		SSR_rig.objAlign footFKControl boneFoot_FK
		if side == "L" then in coordsys local rotate footFKControl (angleaxis 180 [0,0,-1])
		setTransformLockFlags footFKControl #{7..9}
		local footFKNull = SSR_rig.nullTransform footFKControl
		
		-- Toe FK controller
		-- The L side will be flipped for symmetric posing.
		local toeFKControl = SSR_rig.ringCTL posBall (boneSize*6) alignAxis:"X" 
		toeFKControl.name = charName + "_" + side + "_toe_FK_CTL"
		SSR_rig.objAlign toeFKControl boneFoot_FK
		if side == "L" then in coordsys local rotate toeFKControl (angleaxis 180 [0,0,-1])
		setTransformLockFlags toeFKControl #{1..3,7..9}	

	-----------------------------------------------------------------------------
	--
	--		FK parenting and IK setup
	--
	-----------------------------------------------------------------------------
		
		-- Leg root constraint, the entire leg will be parented to this
		local LegRoot = SSR_rig.makeHelper [0, posHip.y, posHip.z] boneSize (prefix + "RootCon") cross:true
		
		-- Leg root, this node will be parented to the main rig
		local legTop = SSR_rig.makeHelper [0, posHip.y, posHip.z] (boneSize*2) (prefix + "Root") box:true
		SSR_rig.parentCon LegRoot legTop
		
		boneFoot_IK.parent = footIKControl
		IKend.parent = boneAnkleEnd_IK
		
		-- Parent the FK controllers into the hierarchy
		hipControl.parent.parent = LegRoot
		
		boneFoot_FK.parent = footFKControl
		footFKNull.parent = SSR_rig.getChildBone boneShin_FK
		
		boneToe_FK.parent = toeFKControl
		toeFKControl.parent = boneFoot_FK
		
		-- Parent both FK and IK hierarchies to the legTop node
		boneThigh.parent = legRoot
		IKroot.parent = legRoot
		
	-----------------------------------------------------------------------------
	--
	--		Additional helpers that will twist and bank the leg
	--
	-----------------------------------------------------------------------------

		--	Helpers we will need to twist and roll the foot
		local heelTwister = SSR_rig.nullTransform boneFoot_IK
		heelTwister.name = prefix + "heelTwister"
		local toeTwister = SSR_rig.nullTransform boneBall_IK
		toeTwister.name = prefix + "toeTwister"
		
		local legBankOut = SSR_rig.makeHelper posSideOut boneSize (prefix + "bankOut") box:true
		local legBankIn = SSR_rig.makeHelper posSideIn boneSize (prefix + "bankIn") box:true
		SSR_rig.objAlign legBankOut footIKControl
		SSR_rig.objAlign legBankIn footIKControl

		local bankNull = SSR_rig.nullTransform legBankOut
		bankNull.parent = toeTwister
		legBankIn.parent = legBankOut
		
		local ballNull = SSR_rig.nullTransform boneBall_IK
		ballNull.parent = legBankIn	

	-----------------------------------------------------------------------------
	--
	--		IK foot following torso/world constraint setup
	--
	-----------------------------------------------------------------------------	
		
		-- Helpers that will be aligned with local or world
		local legWorld = SSR_rig.makeHelper posAnkle boneSize (prefix + "footConstraint_World") box:true
		local legLocal = SSR_rig.makeHelper posAnkle (boneSize*1.5) (prefix + "footConstraint_Local") box:true
		SSR_rig.objAlign legWorld footIKControl
		SSR_rig.objAlign legLocal footIKControl
		legLocal.parent = LegRoot
		
		-- Null that the controller will be parented to, and that will be constrained
		local legIKNull = SSR_rig.makeHelper posAnkle boneSize (prefix + "footConstraint") cross:true
		SSR_rig.objAlign legIKNull footIKControl
		footIKControl.parent = legIKNull
		legIKNull.parent = LegRoot
		
		-- The constraint controllers
		local legCon = SSR_rig.posConTwo legIKNull legLocal legWorld
		local legRotCon = SSR_rig.rotConTwo legIKNull legLocal	legWorld
		

	-----------------------------------------------------------------------------
	--
	--		FK/IK Switch and constraints
	--
	-----------------------------------------------------------------------------	
		
		local hipSwitchCon  = SSR_rig.rotConTwo boneThigh boneThigh_IK boneThigh_FK
		local kneeSwitchCon = SSR_rig.rotConTwo boneShin boneShin_IK boneShin_FK
		local toeSwitchCon  = SSR_rig.rotConTwo boneToe boneToe_IK boneToe_FK
		
		hipSwitchCon.weight[1].controller = Bezier_Float()
		kneeSwitchCon.weight[1].controller = Bezier_Float()
		
		-- Expose helper to measure the IK thigh length
		local exposeIKThigh = exposeTM pos:posKnee size:(boneSize*3) name:(prefix + "IK_exposeThigh")
		exposeIKThigh.parent = boneThigh_IK
		exposeIKThigh.useParent = false
		exposeIKThigh.localReferenceNode = boneShin_IK
		exposeIKThigh.exposeNode = boneThigh_IK
		
		-- Expose helper to measure the FK thigh length
		local exposeFKThigh = exposeTM pos:posKnee size:(boneSize*3) name:(prefix + "FK_exposeThigh")
		exposeFKThigh.parent = boneThigh_FK
		exposeFKThigh.useParent = false
		exposeFKThigh.localReferenceNode = boneShin_FK
		exposeFKThigh.exposeNode = boneThigh_FK
		
		-- Expression controller to adjust the skinned thigh length
		local thighStretchCon = Float_Expression()
		boneShin.Position.controller.X_position.controller = thighStretchCon
		thighStretchCon.AddScalarTarget "ik_len" exposeIKThigh.distance.controller
		thighStretchCon.AddScalarTarget "fk_len" exposeFKThigh.distance.controller
		thighStretchCon.AddScalarTarget "weight" boneThigh.rotation.controller.weight[1].controller
		thighStretchCon.SetExpression "(ik_len*(weight*0.01))+(fk_len*((100-weight)*0.01))"
		
		-- Expose helper to measure the IK shin length
		local exposeIKShin = exposeTM pos:posAnkle size:(boneSize*3) name:(prefix + "IK_exposeShin")
		exposeIKShin.parent = boneShin_IK
		exposeIKShin.useParent = false
		exposeIKShin.localReferenceNode = boneAnkleEnd_IK
		exposeIKShin.exposeNode = boneShin_IK
		
		-- Expose helper to measure the FK shin length
		local exposeFKShin = exposeTM pos:posAnkle size:(boneSize*3) name:(prefix + "FK_exposeShin")
		exposeFKShin.parent = boneShin_FK
		exposeFKShin.useParent = false
		exposeFKShin.localReferenceNode = boneFoot_FK
		exposeFKShin.exposeNode = boneShin_FK
		
		-- Expression controller to adjust the skinned thigh length
		local shinStretchCon = Float_Expression()
		boneFoot.Position.controller.X_position.controller = shinStretchCon
		shinStretchCon.AddScalarTarget "ik_len" exposeIKShin.distance.controller
		shinStretchCon.AddScalarTarget "fk_len" exposeFKShin.distance.controller
		shinStretchCon.AddScalarTarget "weight" boneShin.rotation.controller.weight[1].controller
		shinStretchCon.SetExpression "(ik_len*(weight*0.01))+(fk_len*((100-weight)*0.01))"

		-- Locked axes are plugged with an empty expression controller (thanks Eigil!)
		local emptyExpCon = Float_Expression()
		emptyExpCon.SetExpression "0"
		kneeFKControl.Position.controller.Y_position.controller = \
		kneeFKControl.Position.controller.Z_position.controller = \
		footFKControl.Position.controller.Y_position.controller = \
		footFKControl.Position.controller.Z_position.controller = \
			emptyExpCon
		
		-- The IK foot goes in the reverse direction. 
		-- We will use a helper object to align to instead
		local footIKAlign = SSR_rig.makeHelper posAnkle boneSize (prefix + "FKIK_footSwitch") box:true
		SSR_rig.objAlign footIKAlign boneFoot_FK
		footIKAlign.parent = boneAnkle_IK
		local footSwitchCon = SSR_rig.rotConTwo boneFoot footIKAlign boneFoot_FK
		

	-----------------------------------------------------------------------------
	--
	--		IK Foot attributes
	--
	-----------------------------------------------------------------------------
			
		-- Xform to scale the FK and IK controllers up and down, depending on which one is weighted
		local fkScale = xform name:"fk_scale"
		local ikScale = xform name:"ik_scale"
			
		-- Add scale modifiers to the pre-created FK and IK controllers
		addModifier kneeLookatControl ikScale before:1
		addModifier kneePinControl ikScale before:1
		addModifier hipControl fkScale before:1
		addModifier kneeFKControl fkScale before:1
		
			
		local ca_IK = attributes transforms
		(
			parameters params rollout:footR
			(
				X_Pos	type:#float ui:PXSp
				Y_Pos	type:#float ui:PYSp
				Z_Pos	type:#float ui:PZSp
				X_Rot	type:#angle ui:RXSp
				Y_Rot	type:#angle ui:RYSp
				Z_Rot	type:#angle ui:RZSp
				soft_IK		type:#float ui:softSp
				legPos_WL	type:#float ui:(legSp, legSl)
			)
			rollout footR "Foot Transforms"
			(
				spinner PXSp 	"Translate X " 	range:[-1000000, 1000000, 0]
				spinner PYSp 	"Translate Y " 	range:[-1000000, 1000000, 0]
				spinner PZSp 	"Translate Z " 	range:[-1000000, 1000000, 0]
				spinner RXSp 	"Rotation X " 	range:[-1000000, 1000000, 0]
				spinner RYSp 	"Rotation Y " 	range:[-1000000, 1000000, 0]
				spinner RZSp 	"Rotation Z " 	range:[-1000000, 1000000, 0]
				spinner softSp	"Soft IK "		range:[0.0, 1000.0, 0]
				label lbl02 	"Foot follows World <> Hip" align:#centre offset:[0,10]
				spinner legSp 	"" range:[0, 100, 0] fieldwidth:40 align:#left offset:[10, 10] across:2
				slider legSl 	"" range:[0, 100, 0]
			)

		)
		
		local ca_Foot = attributes footControls
		(
			parameters params2 rollout:attR
			(
				footRoll 	type:#angle ui:footRollSp
				ballRoll	type:#angle ui:ballRollSp
				sideRoll	type:#angle ui:sideRollSp
				toeWiggle	type:#angle ui:toeWiggleSp
				heelTwist	type:#angle ui:heelTwistSp
				toeTwist 	type:#angle ui:toeTwistSp
			)
			rollout attR "Foot Attributes"
			(
				spinner footRollSp 		"Foot Roll " 	range:[-90, 90, 0]
				spinner ballRollSp		"Ball Roll "	range:[-90, 90, 0]
				spinner sideRollSp		"Foot Bank "	range:[-90, 90, 0]
				spinner toeWiggleSp 	"Toes Wiggle " 	range:[-90, 90, 0]
				spinner heelTwistSp		"Heel Twist "	range:[-90, 90, 0]
				spinner toeTwistSp		"Toe Twist "	range:[-90, 90, 0]
			)
		)		
		
		-- Leg FK/IK switch
		local ca_SWITCH = attributes FKIK
		(
			parameters params rollout:fkik
			(
				FK_IK_switch 	type:#float ui:(IKSp, IKSl)	
			)

			rollout fkik "FK IK Switch"
			(
				label lbl02 	"Set to IK <> FK" align:#centre offset:[0,5]
				spinner IKSp	"" range:[0, 100, 0] fieldwidth:40 align:#left offset:[10, 10] across:2
				slider IKSl 	"" range:[0, 100, 0]			
			)
		)
		
		SSR_rig.nullTransform footIKControl
		
		-- Add custom attributes for the controller
		local penMod_IK = 'PEN_Attribute_Holder 2'()
		addModifier footIKControl ikScale
		addModifier footIKControl penMod_IK
		custAttributes.add penMod_IK ca_IK
		custAttributes.add penMod_IK ca_Foot
		custAttributes.add penMod_IK ca_SWITCH
		
		-- Transform connections
		penMod_IK.transforms.X_Pos.controller = \
		footIKControl.pos.controller.X_Position.controller
		penMod_IK.transforms.Y_Pos.controller = \
		footIKControl.pos.controller.Y_Position.controller
		penMod_IK.transforms.Z_Pos.controller = \
		footIKControl.pos.controller.Z_Position.controller
		
		penMod_IK.transforms.X_Rot.controller = \
		footIKControl.rotation.controller.X_Rotation.controller
		penMod_IK.transforms.Y_Rot.controller = \
		footIKControl.rotation.controller.Y_Rotation.controller
		penMod_IK.transforms.Z_Rot.controller = \
		footIKControl.rotation.controller.Z_Rotation.controller
		
		-- Soft IK
		penMod_IK.transforms.soft_IK.controller = \
			limitCon[#Limits][#Upper_Smoothing].controller = \
			Bezier_float()
		
		-- Leg follow hip or world connections
		paramWire.connect penMod_IK.transforms[#legPos_WL] \
			legIKNull.position.controller[#Position_Weight_0] \
			"legPos_WL"
		paramWire.connect penMod_IK.transforms[#legPos_WL] \
			legIKNull.position.controller[#Position_Weight_1] \
			"100-legPos_WL"
		paramWire.connect penMod_IK.transforms[#legPos_WL] \
			legIKNull.rotation.controller[#Orientation_Weight_0] \
			"legPos_WL"
		paramWire.connect penMod_IK.transforms[#legPos_WL] \
			legIKNull.rotation.controller[#Orientation_Weight_1] \
			"100-legPos_WL"
		
		-- Freeze transform on foot, ball and ankle
		SSR_rig.freezeTransform heelTwister
		SSR_rig.freezeTransform toeTwister
		SSR_rig.freezeTransform boneFoot_IK
		SSR_rig.freezeTransform boneBall_IK
		SSR_rig.freezeTransform boneAnkle_IK
		SSR_rig.freezeTransform boneToe_IK

		-- Set up the necessary wiring for the foot
		-- Foot roll (first heel, then toe)
		paramWire.connect penMod_IK.footControls[#footRoll] \
			heelTwister.rotation.controller.Zero_Euler_XYZ.controller[#Z_Rotation] \
			"if footRoll < 0 then -footRoll else 0"
		
		paramWire.connect penMod_IK.footControls[#footRoll] \
			toeTwister.rotation.controller.Zero_Euler_XYZ.controller[#Z_Rotation] \
			"if footRoll > 0 then -footRoll else 0"
		
		-- Ball roll
		paramWire.connect penMod_IK.footControls[#ballRoll] \
			boneAnkle_IK.rotation.controller.Zero_Euler_XYZ.controller[#Z_Rotation] \
			"-ballRoll"
		
		-- Foot side roll (first outside, then inside)
		if side == "R" then
		(
			paramWire.connect penMod_IK.footControls[#sideRoll] \
				legBankOut.rotation.controller[#Y_Rotation] \
				"if sideRoll < 0 then sideRoll else 0"
			
			paramWire.connect penMod_IK.footControls[#sideRoll] \
				legBankIn.rotation.controller[#Y_Rotation] \
				"if sideRoll > 0 then sideRoll else 0"
		) else (
			paramWire.connect penMod_IK.footControls[#sideRoll] \
				legBankOut.rotation.controller[#Y_Rotation] \
				"if sideRoll > 0 then sideRoll else 0"
			
			paramWire.connect penMod_IK.footControls[#sideRoll] \
				legBankIn.rotation.controller[#Y_Rotation] \
				"if sideRoll < 0 then sideRoll else 0"	
		)
		
		-- Heel Twist
		paramWire.connect penMod_IK.footControls[#heelTwist] \
			heelTwister.rotation.controller.Zero_Euler_XYZ.controller[#Y_Rotation] \
			"heelTwist"
		
		-- Toe Twist
		paramWire.connect penMod_IK.footControls[#toeTwist] \
			toeTwister.rotation.controller.Zero_Euler_XYZ.controller[#Y_Rotation] \
			"toeTwist"

		-- Toe wiggle
		paramWire.connect penMod_IK.footControls[#toeWiggle] \
			boneToe_IK.rotation.controller.Zero_Euler_XYZ.controller[#Z_Rotation] \
			"toeWiggle"
		
		-- IK/FK switch connections
		penMod_IK.FKIK.FK_IK_switch.controller = bezier_float()
		
		paramWire.connect penMod_IK.FKIK[#FK_IK_switch] \
			boneThigh.rotation.controller[#Orientation_Weight_0] \
			"100-FK_IK_switch"		
		paramWire.connect penMod_IK.FKIK[#FK_IK_switch] \
			boneThigh.rotation.controller[#Orientation_Weight_1] \
			"FK_IK_switch"
		
		boneToe.rotation.controller.weight[1].controller = boneFoot.rotation.controller.weight[1].controller = \
			boneShin.rotation.controller.weight[1].controller = \
			boneThigh.rotation.controller.weight[1].controller
		boneToe.rotation.controller.weight[2].controller = boneFoot.rotation.controller.weight[2].controller = \
			boneShin.rotation.controller.weight[2].controller = \
			boneThigh.rotation.controller.weight[2].controller
			
		
			
	-----------------------------------------------------------------------------
	--
	--		FK Foot attributes
	--
	-----------------------------------------------------------------------------
		
		local ca_FK = attributes transforms
		(
			parameters params rollout:footR
			(
				X_Rot	type:#angle ui:RXSp
				Y_Rot	type:#angle ui:RYSp
				Z_Rot	type:#angle ui:RZSp
			)
			rollout footR "Leg Transforms"
			(
				spinner RXSp 	"Rotation X " 	range:[-1000000, 1000000, 0]
				spinner RYSp 	"Rotation Y " 	range:[-1000000, 1000000, 0]
				spinner RZSp 	"Rotation Z " 	range:[-1000000, 1000000, 0]
			)
		)
		
		local penMod_FK = 'PEN_Attribute_Holder 2'()
		addModifier footFKControl fkScale
		addModifier footFKControl penMod_FK
		custAttributes.add penMod_FK ca_FK
		custAttributes.add penMod_FK ca_SWITCH
		
		penMod_FK.transforms.X_Rot.controller = \
		footFKControl.rotation.controller.X_Rotation.controller
		penMod_FK.transforms.Y_Rot.controller = \
		footFKControl.rotation.controller.Y_Rotation.controller
		penMod_FK.transforms.Z_Rot.controller = \
		footFKControl.rotation.controller.Z_Rotation.controller
		
		penMod_FK.FKIK.FK_IK_switch.controller = penMod_IK.FKIK.FK_IK_switch.controller
		
		footFKControl.modifiers[1].transforms.X_Rot = 0.0
		footFKControl.modifiers[1].transforms.Y_Rot = 0.0
		footFKControl.modifiers[1].transforms.Z_Rot = 0.0
		
		-- Xforms are used to scale down the FK and IK controls when not used
		paramWire.connect penMod_IK.FKIK[#FK_IK_switch] \
			fkScale.Gizmo.controller[#Scale] \
			"[FK_IK_switch, FK_IK_switch, FK_IK_switch]*0.01"
		paramWire.connect penMod_IK.FKIK[#FK_IK_switch] \
			ikScale.Gizmo.controller[#Scale] \
			"[1,1,1]-[FK_IK_switch, FK_IK_switch, FK_IK_switch]*0.01"
		
	-----------------------------------------------------------------------------
	--
	--		FK toe attributes
	--
	-----------------------------------------------------------------------------

		local ca_T_FK = attributes transforms
		(
			parameters params rollout:footR
			(
				X_Rot	type:#angle ui:RXSp
				Y_Rot	type:#angle ui:RYSp
				Z_Rot	type:#angle ui:RZSp
			)
			rollout footR "Toe Transforms"
			(
				spinner RXSp 	"Rotation X " 	range:[-1000000, 1000000, 0]
				spinner RYSp 	"Rotation Y " 	range:[-1000000, 1000000, 0]
				spinner RZSp 	"Rotation Z " 	range:[-1000000, 1000000, 0]
			)
		)
		
		SSR_rig.nullTransform toeFKControl
		
		penMod_T_FK = 'PEN_Attribute_Holder 2'()
		addModifier toeFKControl fkScale
		addModifier toeFKControl penMod_T_FK
		custAttributes.add penMod_T_FK ca_T_FK
		
		penMod_T_FK.transforms.X_Rot.controller = \
			toeFKControl.rotation.controller.X_Rotation.controller
		penMod_T_FK.transforms.Y_Rot.controller = \
			toeFKControl.rotation.controller.Y_Rotation.controller
		penMod_T_FK.transforms.Z_Rot.controller = \
			toeFKControl.rotation.controller.Z_Rotation.controller
		
		toeFKControl.modifiers[1].transforms.X_Rot = 0.0
		toeFKControl.modifiers[1].transforms.Y_Rot = 0.0
		toeFKControl.modifiers[1].transforms.Z_Rot = 0.0
		

	-----------------------------------------------------------------------------
	--
	--		FK hip attributes
	--
	-----------------------------------------------------------------------------
	/*

	--	Shoulder attributes
		local ca_S = attributes constraints
		(
			parameters params rollout:shoulder
			(
				shoulderRot_WL 	type:#float ui:(shoulderRotSp, shoulderRotSl)
			)
			
			rollout shoulder "Shoulder Constraints"
			(
				label lbl02 		"Shoulder orients Clavicle <> World" align:#centre offset:[0,10]
				spinner shoulderRotSp 	"" range:[0, 100, 0] fieldwidth:40 align:#left offset:[10, 10] across:2
				slider shoulderRotSl 	"" range:[0, 100, 0]
			)
		)
		
		-- Add custom attributes to the shoulder controller
		local penMod_S = shoulderControl.modifiers[1]
		addModifier shoulderControl fkScale before:1
		custAttributes.add penMod_S ca_S
		custAttributes.add penMod_S ca_IK

		-- Shoulder connections
		paramWire.connect penMod_S.constraints[#shoulderRot_WL] \
			shoulderNull.rotation.controller[#Orientation_Weight_0] \
			"shoulderRot_WL"
		paramWire.connect penMod_S.constraints[#shoulderRot_WL] \
			shoulderNull.rotation.controller[#Orientation_Weight_1] \
			"100-shoulderRot_WL"
			
	*/

	-----------------------------------------------------------------------------
	--
	--		Twist Bones
	--
	-----------------------------------------------------------------------------
		
		if doTwist == true then
		(
			twistReturn = SSR_rig.stretchTwistBones boneShin boneFoot numTwistShin twistUp:false debug:debug
			
			twistReturn = SSR_rig.stretchTwistBones boneThigh (boneThigh.parent) numTwistThigh twistUp:true debug:debug

			-- The previous skinBones are no longer skinning bones
			boneThigh.name = prefix + "bone_Thigh"
			boneShin.name = prefix + "bone_Shin"
		)			
		
		return #(footIKControl, footFKControl, toeFKControl, hipControl, kneeFKControl, kneeLookatControl)
		
	)
)

SSR_part = SSR_part()