-----------------------------------------------------------------------------
--
--		Root setup
--
--		v1.0: Standard setup
--		v1.1: Named all objects and put them into layers
--		v1.2: New connection points for arm and leg into spine
--		v1.3:	Scale disabled
--
-----------------------------------------------------------------------------
global SSR_part

struct SSR_part
(
	function doPart charName extParams:#(true) boneSize:1 =
	(
		
		local doLocks = extParams[1]

		-----------------------------------------------------------------------------
		--
		--		Create Root controller
		--
		-----------------------------------------------------------------------------	
			
		-- Create root controller
		local rootControl = SSR_rig.ringCTL [0,0,0] (boneSize*50) alignAxis:"Z"	
		rootControl.name = charName + "_root_CTL"
		
		-- Custom attributes for the root controller
		local rootMod = 'PEN_Attribute_Holder 2'()
		addModifier rootControl rootMod
		
		local rootAtt = attributes transforms
		(
			parameters params rollout:handR
			(
				X_Pos		type:#float ui:X_PosSp
				Y_Pos		type:#float ui:Y_PosSp
				Z_Pos		type:#float ui:Z_PosSp
				X_Rot		type:#angle ui:X_RotSp
				Y_Rot		type:#angle ui:Y_RotSp
				Z_Rot		type:#angle ui:Z_RotSp
				Scale_Rig	type:#float ui:SXSp
			)
			rollout handR "Transforms"
			(
				spinner X_PosSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_PosSp 	"Translate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_PosSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
				spinner X_RotSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_RotSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_RotSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
				spinner SXSp 		"Rig Scale " 	range:[-1000000, 1000000, 1.0] align:#right enabled:false
			)
		)		
		
		custAttributes.add rootMod rootAtt
		
		-- Connect up custom attributes
		rootMod.transforms.X_Pos.controller = \
		rootControl.pos.controller.X_Position.controller
		rootMod.transforms.Y_Pos.controller = \
		rootControl.pos.controller.Y_Position.controller
		rootMod.transforms.Z_Pos.controller = \
		rootControl.pos.controller.Z_Position.controller

		rootMod.transforms.X_Rot.controller = \
		rootControl.rotation.controller.X_Rotation.controller
		rootMod.transforms.Y_Rot.controller = \
		rootControl.rotation.controller.Y_Rotation.controller
		rootMod.transforms.Z_Rot.controller = \
		rootControl.rotation.controller.Z_Rotation.controller
		
		rootControl.scale.controller = ScaleXYZ ()
		
		rootMod.transforms.Scale_Rig.controller = \
		rootControl.scale.controller.X_Scale.controller = \
		rootControl.scale.controller.Y_Scale.controller = \
		rootControl.scale.controller.Z_Scale.controller
		
		setTransformLockFlags rootControl #{7..9}
		
		-----------------------------------------------------------------------------
		--
		--		Create Locks 
		--
		-----------------------------------------------------------------------------	
			
	if doLocks == true then
	(
		
		-- We will label the controllers with a substring of the character name
		local conText = ""
		if charName.count > 5 then conText = toUpper ( substring charName 1 5 )
			else conText = toUpper charName
		
		-- First lock controller
		local lockControl_1 = SSR_rig.crossCTL [0, 0, 0] (boneSize*10.0)
		lockControl_1.name = charName + "_rootLock1_CTL"
		
		local label1 = text text:conText size:(boneSize*8.0)
		local label2 = text text:"root lock 1" size:(boneSize*6.0)
		
		in coordsys world rotate label1 (angleaxis 90 [1,0,0])
		label1.pos = [0,0,boneSize*12.0]
		in coordsys world rotate label2 (angleaxis 90 [1,0,0])
		label2.pos = [0,0,boneSize*6.0]
		
		convertTo label1 SplineShape
		convertTo label2 SplineShape
		
		addAndWeld lockControl_1 label1 -1
		addAndWeld lockControl_1 label2 -1
		
		setTransformLockFlags lockControl_1 #{7..9}
		
		-- Second lock controller
		local lockControl_2 = SSR_rig.crossCTL [0, 0, 0] (boneSize*10.0) 
		lockControl_2.name = charName + "_rootLock2_CTL"
		
		local label1 = text text:conText size:(boneSize*8.0)
		local label2 = text text:"root lock 2" size:(boneSize*6.0)
		
		in coordsys world rotate label1 (angleaxis 90 [1,0,0])
		label1.pos = [0,0,boneSize*12.0]
		in coordsys world rotate label2 (angleaxis 90 [1,0,0])
		label2.pos = [0,0,boneSize*6.0]
		
		convertTo label1 SplineShape
		convertTo label2 SplineShape
		
		addAndWeld lockControl_2 label1 -1
		addAndWeld lockControl_2 label2 -1
		
		setTransformLockFlags lockControl_2 #{7..9}
		
		-- World lock node
		local lockWorld = SSR_rig.makeHelper [0,0,0] (boneSize*10.0) (charName + "_lockWorld") cross:true
		
		-- The node we will be constraining to the three above
		local lockNode = SSR_rig.makeHelper [0,0,0] (boneSize*10.0) (charName + "_rootParent") box:true
		
		-- Position and location constraints
		local posCon = Position_Constraint()
		posCon.appendTarget lockWorld 100
		posCon.appendTarget lockControl_1 0
		posCon.appendTarget lockControl_2 0
		lockNode.pos.controller = posCon
		posCon.relative = on
			
		local rotCon = Orientation_Constraint()
		rotCon.appendTarget lockWorld 100
		rotCon.appendTarget lockControl_1 0
		rotCon.appendTarget lockControl_2 0
		lockNode.rotation.controller = rotCon
		rotCon.relative = on
		
		rootControl.parent = lockNode
		
		-- Transform attributes for the lock controllers
		local att = attributes transforms
		(
			parameters params rollout:handR
			(
				X_Pos		type:#float ui:X_PosSp
				Y_Pos		type:#float ui:Y_PosSp
				Z_Pos		type:#float ui:Z_PosSp
				X_Rot		type:#angle ui:X_RotSp
				Y_Rot		type:#angle ui:Y_RotSp
				Z_Rot		type:#angle ui:Z_RotSp
			)
			rollout handR "Transforms"
			(
				spinner X_PosSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_PosSp 	"Translate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_PosSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
				spinner X_RotSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_RotSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_RotSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	
		
		local mod_1 = 'PEN_Attribute_Holder 2'()
		addModifier lockControl_1 mod_1
		custAttributes.add mod_1 att
		
		local mod_2 = 'PEN_Attribute_Holder 2'()
		addModifier lockControl_2 mod_2
		custAttributes.add mod_2 att
		
		-- Connections for the lock controller 1
		mod_1.transforms.X_Pos.controller = \
			lockControl_1.pos.controller.X_Position.controller
		mod_1.transforms.Y_Pos.controller = \
			lockControl_1.pos.controller.Y_Position.controller
		mod_1.transforms.Z_Pos.controller = \
			lockControl_1.pos.controller.Z_Position.controller
		
		mod_1.transforms.X_Rot.controller = \
			lockControl_1.rotation.controller.X_Rotation.controller
		mod_1.transforms.Y_Rot.controller = \
			lockControl_1.rotation.controller.Y_Rotation.controller
		mod_1.transforms.Z_Rot.controller = \
			lockControl_1.rotation.controller.Z_Rotation.controller
		
		-- Connections for the lock controller 2
		mod_2.transforms.X_Pos.controller = \
			lockControl_2.pos.controller.X_Position.controller
		mod_2.transforms.Y_Pos.controller = \
			lockControl_2.pos.controller.Y_Position.controller
		mod_2.transforms.Z_Pos.controller = \
			lockControl_2.pos.controller.Z_Position.controller
		
		mod_2.transforms.X_Rot.controller = \
			lockControl_2.rotation.controller.X_Rotation.controller
		mod_2.transforms.Y_Rot.controller = \
			lockControl_2.rotation.controller.Y_Rotation.controller
		mod_2.transforms.Z_Rot.controller = \
			lockControl_2.rotation.controller.Z_Rotation.controller
		
		-- Lock attributes for the root controller
		local lockAtt = attributes locks
		(
			parameters params rollout:handR
			(
				lockNum		type:#integer ui:lockNum_Sp
			)
			rollout handR "Locks"
			(
				label lbl1 "World    Lock 1    Lock 2" align:#center offset:[-10,0]
				slider lockNum_Sp "" range:[0, 2, 0] type:#integer orient:#horizontal ticks:0 align:#center width:100.0
			)
		)		
		
		custAttributes.add rootMod lockAtt
		
		-- Connections for the lock attributes
		paramWire.connect rootMod.locks[#lockNum] \
			lockNode.pos.controller[#Position_Weight_0] \
			"if lockNum==0 then 100 else 0"
		
		paramWire.connect rootMod.locks[#lockNum] \
			lockNode.pos.controller[#Position_Weight_1] \
			"if lockNum==1 then 100 else 0"
		
		paramWire.connect rootMod.locks[#lockNum] \
			lockNode.pos.controller[#Position_Weight_2] \
			"if lockNum==2 then 100 else 0"
		
		lockNode.rotation.controller[#Orientation_Weight_0].controller = lockNode.position.controller[#Position_Weight_0].controller
		lockNode.rotation.controller[#Orientation_Weight_1].controller = lockNode.position.controller[#Position_Weight_1].controller
		lockNode.rotation.controller[#Orientation_Weight_2].controller = lockNode.position.controller[#Position_Weight_2].controller
		
		local rigLayerName = charName + " Rig"
		local rigLayer = layermanager.getlayerfromname rigLayerName
		if rigLayer == undefined then rigLayer = layermanager.newLayerfromname rigLayerName
		
		rigLayer.addNode lockWorld
		rigLayer.addNode lockNode
		
	) -- if doLocks == true then
	
	-----------------------------------------------------------------------------
	--
	--		Connecting everything up
	--
	-----------------------------------------------------------------------------	

		-- Connect spine to root
		try
		(
			execute ("$'" + charName + "_torso_CTL_null'.parent = $'" + charName + "_root_CTL'")
		)
		catch SSR_gen.verbose "Could not connect spine!"

		-- Connect right leg to spine and root
		try 
		(
			execute ("$'" + charName + "_R_leg_Root'.parent = $'" + charName + "_hip_CTL'")
			execute ("$'" + charName + "_R_leg_RootCon'.parent = $'" + charName + "_root_CTL'")
			execute ("$'" + charName + "_R_leg_footConstraint_World'.parent = $'" + charName + "_root_CTL'")
		)
		catch SSR_gen.verbose "Could not connect right leg!"
			
		-- Connect left leg to spine and root
		try 
		(
			execute ("$'" + charName + "_L_leg_Root'.parent = $'" + charName + "_hip_CTL'")
			execute ("$'" + charName + "_L_leg_RootCon'.parent = $'" + charName + "_root_CTL'")
			execute ("$'" + charName + "_L_leg_footConstraint_World'.parent = $'" + charName + "_root_CTL'")
		)
		catch SSR_gen.verbose "Could not connect left leg!"
		
		-- Connect right arm to spine
		try 
		(
			execute ("$'" + charName + "_R_arm_Root'.parent = $'" + charName + "_spine_clavicleParent'")
			execute ("$'" + charName + "_R_arm_RootCon'.parent = $'" + charName + "_root_CTL'")
			execute ("$'" + charName + "_R_arm_handConstraint_World'.parent = $'" + charName + "_root_CTL'")
			execute ("$'" + charName + "_R_arm_elbowConstraint_World'.parent = $'" + charName + "_root_CTL'")
		)
		catch SSR_gen.verbose "Could not connect right arm!"
		
		-- Connect left arm to spine
		try 
		(
			execute ("$'" + charName + "_L_arm_Root'.parent = $'" + charName + "_spine_clavicleParent'")
			execute ("$'" + charName + "_L_arm_RootCon'.parent = $'" + charName + "_root_CTL'")
			execute ("$'" + charName + "_L_arm_handConstraint_World'.parent = $'" + charName + "_root_CTL'")
			execute ("$'" + charName + "_L_arm_elbowConstraint_World'.parent = $'" + charName + "_root_CTL'")
		)
		catch SSR_gen.verbose "Could not connect left arm!"
			
		-- Connect head to spine
		try 
		(
			execute ("$" + charName + "_head_Root.parent = $" + charName + "_spine_bone_ChestEnd")
			execute ("$" + charName + "_head_headConstraint_Root.parent = $" + charName + "_spine_bone_ChestEnd")
			
			execute ("$" + charName + "_head_eyeConstraint_WorldParent.parent = $" + charName + "_root_CTL")
			execute ("$" + charName + "_lookat_CTL.parent = $" + charName + "_root_CTL")
		)
		catch SSR_gen.verbose "Could not connect head!"
			
		-- Parent all World constraints to Root node
		try
		(
			execute ("$'" + charName + "*Constraint_world'.parent = $" + charName + "_root_CTL")
		)
		catch SSR_gen.verbose "Could not connect world constraints to root controller!"
		
		return #(rootControl)
	)
)

SSR_part = SSR_part()