-----------------------------------------------------------------------------
--
--		FK centered spine setup
--
--		v1.0:	Standard setup
--		v1.1:	Renamed attributes
--		v1.2: Optional spine root aligned to world
--
-----------------------------------------------------------------------------
global SSR_part

struct SSR_part
(
	function doPart charName extParams:#(true) boneSize:1 =
	(
		-- External parameters
		local rootAlign = extParams[1]
		
		local helperList = SSR_loc.collectLocators charName ("*Locator_Spine_*")
		local posHip, posLBack, posMBack, posHBack, posChest, posRoot, posChestR, posChestL, posBelly
		local prefix = (charName + "_spine_")
		local doBreathe = false

		-- Look for helper objects that give locations to spine bone positions
		if helperList != false then for h in helperList do
		(
			if matchPattern h.name pattern:("*_Spine_Hip") == true then 	posHip = h.pos
			if matchPattern h.name pattern:("*_Spine_LowBack") == true then posLBack = h.pos
			if matchPattern h.name pattern:("*_Spine_MidBack") == true then posMBack = h.pos
			if matchPattern h.name pattern:("*_Spine_UpperBack") == true then posHBack = h.pos
			if matchPattern h.name pattern:("*_Spine_Chest") == true then posChest = h.pos
			if matchPattern h.name pattern:("*_Leg_R_Hip") == true then posRoot = [0, h.pos.y, h.pos.z]
			if matchPattern h.name pattern:("*_Chest_R_Breathe") == true then posChestR = h.pos
			if matchPattern h.name pattern:("*_Chest_L_Breathe") == true then posChestL = h.pos
			if matchPattern h.name pattern:("*_Belly_Breathe") == true then posBelly = h.pos
		)
		
		-- If one of these helpers are not found, we will exit
		if posHip == undefined or posLBack == undefined \
			or posMBack == undefined or posHBack == undefined or posChest == undefined \
		then (
			SSR_gen.verbose ("Missing locators for spine, operation aborted.")
			return false
		) else (
			SSR_gen.verbose ("All locators are found, now creating spine.")
		)
		
		-- If all chest breathe attributes are found, we will add breathe attributes
		if posChestR != undefined and posChestL != undefined and posBelly != undefined then 
			doBreathe = true
		
		
	-----------------------------------------------------------------------------
	--
	--		Spline IK and Bones
	--
	-----------------------------------------------------------------------------
		
		-- Create our main bones
		--local boneSize = (distance posHip poschest) / 20
		local zAxis = [ -1, 0, 0]
		
		-- Downward bones chain
		local boneHip		= SSR_rig.makeBone (prefix + "skinBone_Hip") posLBack posHip zAxis boneSize
		local boneHipEnd	= SSR_rig.makeBone (prefix + "bone_HipEnd") posHip (posHip + [0, 0, boneSize]) zAxis boneSize
		local boneLBack		= SSR_rig.makeBone (prefix + "skinBone_LowerBack") posMBack posLBack zAxis boneSize
		
		-- Upward bones chain
		local boneUBack		= SSR_rig.makeBone (prefix + "skinBone_UpperBack") posMBack posHBack zAxis boneSize
		local boneChest		= SSR_rig.makeBone (prefix + "skinBone_Chest") posHBack poschest zAxis boneSize
		local boneChestEnd	= SSR_rig.makeBone (prefix + "bone_ChestEnd") poschest (poschest + [0, 0, boneSize]) zAxis boneSize
		
		boneHip.frontfin 	= true
		boneLBack.frontfin 	= true
		boneUBack.frontfin 	= true
		boneChest.frontfin 	= true
		
		-- Alignments and parenting
		SSR_rig.objAlign boneChestEnd boneChest
		SSR_rig.objAlign boneHipEnd boneHip
		boneChestEnd.parent = boneChest
		boneHipEnd.parent = boneHip
		boneChest.parent = boneUBack
		boneHip.parent = boneLBack
		
	-----------------------------------------------------------------------------
	--
	--		Breathe Bones and helpers
	--
	-----------------------------------------------------------------------------	
		if doBreathe == true then
		(
			-- Chest breathers
			local breatheRRoot 		= posChestR - (normalize (posChestR - posHBack) * boneSize * 2)
			local boneBreatheR		= SSR_rig.makeBone (prefix + "skinBone_BreatheChestR") breatheRRoot posChestR [0,0,1] boneSize
			local boneBreatheREnd	= SSR_rig.endBone boneBreatheR
			boneBreatheREnd.name 	= prefix + "bone_BreatheChestREnd"
			
			local breatheLRoot 		= posChestL - (normalize (posChestL - posHBack) * boneSize * 2)
			local boneBreatheL		= SSR_rig.makeBone (prefix + "skinBone_BreatheChestL") breatheLRoot posChestL [0,0,1] boneSize
			local boneBreatheLEnd	= SSR_rig.endBone boneBreatheL
			boneBreatheLEnd.name 	= prefix + "bone_BreatheChestLEnd"
			
			local breatheRNull 	= SSR_rig.nullTransform boneBreatheR
			breatheRNull.name 	= substituteString (breatheRNull.name) "_skinBone_" "_"
			local breatheLNull 	= SSR_rig.nullTransform boneBreatheL
			breatheLNull.name 	= substituteString (breatheLNull.name) "_skinBone_" "_"
			
			-- Chest breathers are constrained between chest bone and upper back bone
			local chestBreatheConUp = SSR_rig.makeHelper [0, posChestL.y, posChestL.z] (boneSize*1.2) (prefix + "breatheCon_Up") box:true
			local chestBreatheConDown = SSR_rig.makeHelper [0, posChestL.y, posChestL.z] boneSize (prefix + "breatheCon_Down") box:true
			local chestBreatheCon = SSR_rig.makeHelper [0, posChestL.y, posChestL.z] boneSize (prefix + "breatheCon") cross:true
			
			chestBreatheConUp.parent = boneChest
			chestBreatheConDown.parent = boneUBack
			
			local chestPosCon = SSR_rig.posConTwo chestBreatheCon chestBreatheConUp chestBreatheConDown
			local chestRotCon = SSR_rig.rotConTwo chestBreatheCon chestBreatheConUp chestBreatheConDown
			
			breatheRNull.parent = breatheLNull.parent = chestBreatheCon
			
			-- Belly breather
			local breatheBellyRoot = posBelly - (normalize (posBelly - posLBack) * boneSize * 2)
			local boneBreatheBelly		= SSR_rig.makeBone (prefix + "skinBone_BreatheBelly") breatheBellyRoot posBelly [0,0,1] boneSize
			local boneBreatheBellyEnd	= SSR_rig.endBone boneBreatheBelly
			boneBreatheBellyEnd.name = prefix + "bone_BreatheBellyEnd"
			
			local breatheBellyNull = SSR_rig.nullTransform boneBreatheBelly
			breatheBellyNull.name = substituteString (breatheBellyNull.name) "_skinBone_" "_"
			
			-- Belly breather is constrained between chest bone and upper back bone
			local bellyBreatheConUp = SSR_rig.makeHelper posBelly (boneSize*1.2) (prefix + "bellyBreatheCon_Up") box:true
			local bellyBreatheConDown = SSR_rig.makeHelper posBelly boneSize (prefix + "bellyBreatheCon_Down") box:true
			local bellyBreatheCon = SSR_rig.makeHelper posBelly boneSize (prefix + "bellyBreatheCon") cross:true
			
			bellyBreatheConUp.parent = boneLBack
			bellyBreatheConDown.parent = boneHip
			
			local bellyPosCon = SSR_rig.posConTwo bellyBreatheCon bellyBreatheConUp bellyBreatheConDown
			local bellyRotCon = SSR_rig.rotConTwo bellyBreatheCon bellyBreatheConUp bellyBreatheConDown
			
			breatheBellyNull.parent = bellyBreatheCon	
			
			skinLayer.addNode boneBreatheR
			skinLayer.addNode boneBreatheL
			skinLayer.addNode boneBreatheBelly
		) -- if doBreathe == true then
		
		-- Clavicle breather
		-- This will be created anyway, as this is the default connection point for the arms to the chest
		local clavicleBreathe = SSR_rig.makeHelper (posChest + [0,-boneSize,0]) boneSize (prefix + "clavicleParent") cross:true
		SSR_rig.objAlign clavicleBreathe boneChest
		local clavicleBreatheNull = SSR_rig.nullTransform clavicleBreathe
		clavicleBreatheNull.parent = boneChest
		

	-----------------------------------------------------------------------------
	--
	--		Control objects
	--
	-----------------------------------------------------------------------------

		-- Top level rotation control objects
		local hipControl 	= SSR_rig.ringCTL posHip (boneSize*20) alignAxis:"X"
		hipControl.name		= charName + "_hip_CTL" 
		SSR_rig.objAlign hipControl boneHip
		setTransformLockFlags hipControl #{1..3,7..9}
		hipControl.pivot = posMBack
		
		local chestControl	= SSR_rig.ringCTL posChest (boneSize*20) alignAxis:"X"
		chestControl.name 	= charName + "_chest_CTL"
		SSR_rig.objAlign chestControl boneChest
		setTransformLockFlags chestControl #{1..3,7..9}
		chestControl.pivot = posMBack
		
		local torsoControl 	= SSR_rig.ringCTL posMBack (boneSize*20) alignAxis:"X"
		torsoControl.name = charName + "_torso_CTL"
		SSR_rig.objAlign torsoControl boneLBack closestAxis:rootAlign -- Optional align to world axis
		setTransformLockFlags torsoControl #{7..9}

		-- Lower level position control objects
		local spineHipControl = SSR_rig.pinCTL posHip -(boneSize*7) (boneSize*3)
		spineHipControl.name = charName + "_spine_hip_CTL"
		SSR_rig.objAlign spineHipControl boneHip
		setTransformLockFlags spineHipControl #{4..9}
		
		local spineLowControl = SSR_rig.pinCTL posLBack -(boneSize*7) (boneSize*3)
		spineLowControl.name = charName + "_spine_low_CTL"
		SSR_rig.objAlign spineLowControl boneLBack
		setTransformLockFlags spineLowControl #{4..9}
		
		local spineMidControl = SSR_rig.pinCTL posMBack (boneSize*7) (boneSize*3)
		spineMidControl.name = charName + "_spine_mid_CTL"
		SSR_rig.objAlign spineMidControl boneUBack
		setTransformLockFlags spineMidControl #{4..9}
		
		local spineUpControl = SSR_rig.pinCTL posHBack (boneSize*7) (boneSize*3)
		spineUpControl.name = charName + "_spine_up_CTL"
		SSR_rig.objAlign spineUpControl boneChest
		setTransformLockFlags spineUpControl #{4..9}
		
		local spineChestControl = SSR_rig.pinCTL posChest (boneSize*7) (boneSize*3)
		spineChestControl.name = charName + "_spine_chest_CTL"
		SSR_rig.objAlign spineChestControl boneChest
		setTransformLockFlags spineChestControl #{4..9}
		
		-- The two rotation controllers will be parented under the torso controller
		hipControl.parent = chestControl.parent = torsoControl

		
	-----------------------------------------------------------------------------
	--
	--		Helpers and constraints
	--
	-----------------------------------------------------------------------------	
		
		--local torsoNull = SSR_rig.nullTransform torsoControl
		
		-- These helpers will be constrained to ease off the rotation of the upward (chest) spine controller
		local spineUpEaseStart = SSR_rig.makeHelper posMBack (boneSize*1.2) (prefix + "spineUpEaseStart") box:true
		spineUpEaseStart.parent = torsoControl
		local spineUpEaseEnd = SSR_rig.makeHelper posHBack (boneSize*1.2) (prefix + "spineUpEaseEnd") box:true
		spineUpEaseEnd.parent = spineUpEaseStart
		
		local chestAlign = SSR_rig.makeHelper posChest (boneSize*1.2) (prefix + "spineChestAlign") box:true
		chestAlign.parent = chestControl
		local torsoAlign = SSR_rig.makeHelper posMBack (boneSize*1.2) (prefix + "spineTorsoAlign") box:true
		torsoAlign.parent = torsoControl
		
		local upEaseStartCon = SSR_rig.rotConTwo spineUpEaseStart torsoAlign chestAlign
		local upEaseEndCon = SSR_rig.rotCon spineUpEaseEnd chestAlign
		
		-- Now do the same for the downwards spine
		local spineDownEaseStart = SSR_rig.makeHelper posMBack (boneSize*1.2) (prefix + "spineDownEaseStart") box:true
		spineDownEaseStart.parent = torsoControl
		local spineDownEaseEnd = SSR_rig.makeHelper posLBack (boneSize*1.2) (prefix + "spineDownEaseEnd") box:true
		spineDownEaseEnd.parent = spineDownEaseStart
		
		local hipAlign = SSR_rig.makeHelper posHip (boneSize*1.2) (prefix + "spineHipAlign") box:true
		hipAlign.parent = hipControl
		
		local downEaseStartCon = SSR_rig.rotConTwo spineDownEaseStart torsoAlign hipAlign
		local downEaseEndCon = SSR_rig.rotCon spineDownEaseEnd hipAlign
		
		-- The lower level position controls will be parented to the interpolated helpers
		spineHipControl.parent 		= spineDownEaseEnd
		spineLowControl.parent 		= spineDownEaseStart
		spineMidControl.parent 		= torsoControl
		spineUpControl.parent 		= spineUpEaseStart
		spineChestControl.parent 	= spineUpEaseEnd
		
		-- And with a setup of lookats and position constraints, they will follow these lower level controls
		local chestPosCon = SSR_rig.posCon boneChestEnd spineChestControl
		local chestLookCon = SSR_rig.lookCon boneChest spineChestControl
		chestLookCon.upnode_world = off; chestLookCon.pickUpNode = spineChestControl

		local upPosCon = SSR_rig.posCon boneChest spineUpControl
		local upLookCon = SSR_rig.lookCon boneUBack spineUpControl
		upLookCon.upnode_world = off; upLookCon.pickUpNode = spineUpControl
		
		local midPosConU = SSR_rig.posCon boneUBack spineMidControl
		local midPosConD = SSR_rig.posCon boneLBack spineMidControl

		local lowPosCon = SSR_rig.posCon boneHip spineLowControl
		local lowLookCon = SSR_rig.lookCon boneLBack spineLowControl
		lowLookCon.upnode_world = off; lowLookCon.pickUpNode = spineLowControl
		
		local hipPosCon = SSR_rig.posCon boneHipEnd spineHipControl
		local hipLookCon = SSR_rig.lookCon boneHip spineHipControl
		hipLookCon.upnode_world = off; hipLookCon.pickUpNode = spineHipControl

		
	-----------------------------------------------------------------------------
	--
	--		Controller custom attributes
	--
	-----------------------------------------------------------------------------
		
		-- Add custom attributes for the hip controller
		local hipMod = 'PEN_Attribute_Holder 2'()
		addModifier hipControl hipMod
		
		local hipAtt = attributes transforms
		(
			parameters params rollout:handR
			(
				X_Rot		type:#angle ui:X_RotSp
				Y_Rot		type:#angle ui:Y_RotSp
				Z_Rot		type:#angle ui:Z_RotSp
			)
			rollout handR "Transforms"
			(
				spinner X_RotSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_RotSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_RotSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)		
		custAttributes.add hipMod hipAtt

		-- Connect up custom attributes
		SSR_rig.nullTransform hipControl
		
		hipMod.transforms.X_Rot.controller = \
			hipControl.rotation.controller.X_Rotation.controller
		hipMod.transforms.Y_Rot.controller = \
			hipControl.rotation.controller.Y_Rotation.controller
		hipMod.transforms.Z_Rot.controller = \
			hipControl.rotation.controller.Z_Rotation.controller

		-- Custom attributes for the chest controller
		local chestMod = 'PEN_Attribute_Holder 2'()
		addModifier chestControl chestMod
		
		custAttributes.add chestMod hipAtt
		
		-- Connect up custom attributes
		SSR_rig.nullTransform chestControl
		
		chestMod.transforms.X_Rot.controller = \
			chestControl.rotation.controller.X_Rotation.controller
		chestMod.transforms.Y_Rot.controller = \
			chestControl.rotation.controller.Y_Rotation.controller
		chestMod.transforms.Z_Rot.controller = \
			chestControl.rotation.controller.Z_Rotation.controller
			
		-- Custom attributes for the torso controller
		local torsoMod = 'PEN_Attribute_Holder 2'()
		addModifier torsoControl torsoMod
		
		local torsoAtt = attributes transforms
		(
			parameters params rollout:handR
			(
				X_Pos		type:#float ui:X_PosSp
				Y_Pos		type:#float ui:Y_PosSp
				Z_Pos		type:#float ui:Z_PosSp
				X_Rot		type:#angle ui:X_RotSp
				Y_Rot		type:#angle ui:Y_RotSp
				Z_Rot		type:#angle ui:Z_RotSp
			)
			rollout handR "Transforms"
			(
				spinner X_PosSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_PosSp 	"Translate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_PosSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
				spinner X_RotSp 	"Rotate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_RotSp 	"Rotate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_RotSp 	"Rotate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)		
		
		custAttributes.add torsoMod torsoAtt
		
		-- Connect up custom attributes
		SSR_rig.nullTransform torsoControl
		
		torsoMod.transforms.X_Pos.controller = \
			torsoControl.pos.controller.X_Position.controller
		torsoMod.transforms.Y_Pos.controller = \
			torsoControl.pos.controller.Y_Position.controller
		torsoMod.transforms.Z_Pos.controller = \
			torsoControl.pos.controller.Z_Position.controller

		torsoMod.transforms.X_Rot.controller = \
			torsoControl.rotation.controller.X_Rotation.controller
		torsoMod.transforms.Y_Rot.controller = \
			torsoControl.rotation.controller.Y_Rotation.controller
		torsoMod.transforms.Z_Rot.controller = \
			torsoControl.rotation.controller.Z_Rotation.controller
		
		-- Add custom attributes to the lower level spine controllers
		local spineAtt = attributes transforms
		(
			parameters params rollout:handR
			(
				X_Pos		type:#float ui:X_PosSp
				Y_Pos		type:#float ui:Y_PosSp
				Z_Pos		type:#float ui:Z_PosSp
			)
			rollout handR "Transforms"
			(
				spinner X_PosSp 	"Translate X " 	range:[-1000000, 1000000, 0] align:#right
				spinner Y_PosSp 	"Translate Y " 	range:[-1000000, 1000000, 0] align:#right
				spinner Z_PosSp 	"Translate Z " 	range:[-1000000, 1000000, 0] align:#right
			)
		)	

		-- Attributes for Spine Hip
		local spineHipMod = 'PEN_Attribute_Holder 2'()
		addModifier spineHipControl spineHipMod	
		custAttributes.add spineHipMod spineAtt
		SSR_rig.nullTransform spineHipControl
		
		spineHipMod.transforms.X_Pos.controller = \
			spineHipControl.pos.controller.X_Position.controller
		spineHipMod.transforms.Y_Pos.controller = \
			spineHipControl.pos.controller.Y_Position.controller
		spineHipMod.transforms.Z_Pos.controller = \
			spineHipControl.pos.controller.Z_Position.controller
		
		-- Attributes for Spine Low
		local spineLowMod = 'PEN_Attribute_Holder 2'()
		addModifier spineLowControl spineLowMod	
		custAttributes.add spineLowMod spineAtt
		SSR_rig.nullTransform spineLowControl
		
		spineLowMod.transforms.X_Pos.controller = \
			spineLowControl.pos.controller.X_Position.controller
		spineLowMod.transforms.Y_Pos.controller = \
			spineLowControl.pos.controller.Y_Position.controller
		spineLowMod.transforms.Z_Pos.controller = \
			spineLowControl.pos.controller.Z_Position.controller	
		
		-- Attributes for Spine Mid
		local spineMidMod = 'PEN_Attribute_Holder 2'()
		addModifier spineMidControl spineMidMod	
		custAttributes.add spineMidMod spineAtt
		SSR_rig.nullTransform spineMidControl
		
		spineMidMod.transforms.X_Pos.controller = \
			spineMidControl.pos.controller.X_Position.controller
		spineMidMod.transforms.Y_Pos.controller = \
			spineMidControl.pos.controller.Y_Position.controller
		spineMidMod.transforms.Z_Pos.controller = \
			spineMidControl.pos.controller.Z_Position.controller	
			
		-- Attributes for Spine Up
		local spineUpMod = 'PEN_Attribute_Holder 2'()
		addModifier spineUpControl spineUpMod	
		custAttributes.add spineUpMod spineAtt
		SSR_rig.nullTransform spineUpControl
		
		spineUpMod.transforms.X_Pos.controller = \
			spineUpControl.pos.controller.X_Position.controller
		spineUpMod.transforms.Y_Pos.controller = \
			spineUpControl.pos.controller.Y_Position.controller
		spineUpMod.transforms.Z_Pos.controller = \
			spineUpControl.pos.controller.Z_Position.controller	
			
		-- Attributes for Spine Chest
		local spineChestMod = 'PEN_Attribute_Holder 2'()
		addModifier spineChestControl spineChestMod	
		custAttributes.add spineChestMod spineAtt
		SSR_rig.nullTransform spineChestControl
		
		spineChestMod.transforms.X_Pos.controller = \
			spineChestControl.pos.controller.X_Position.controller
		spineChestMod.transforms.Y_Pos.controller = \
			spineChestControl.pos.controller.Y_Position.controller
		spineChestMod.transforms.Z_Pos.controller = \
			spineChestControl.pos.controller.Z_Position.controller		
		
		if doBreathe == true then
		(
			-- Custom attributes for the breathe bones
			local breatheAtt = attributes breathe_controls
			(
				parameters params rollout:handR
				(
					clavBreathe		type:#float ui:clavSp
					chestBreathe	type:#float ui:chestSp
					bellyBreathe	type:#float ui:bellySp
				)
				rollout handR "Breathe Controls"
				(
					spinner clavSp 		"Clavicle Breathe " range:[-1000000, 1000000, 0] align:#right fieldWidth:50
					spinner chestSp 	"Chest Breathe " 	range:[-1000000, 1000000, 0] align:#right fieldWidth:50
					spinner bellySp 	"Belly Breathe " 	range:[-1000000, 1000000, 0] align:#right fieldWidth:50
					label lbl01			"Hold ALT for fine control" offset:[0,10]
				)
			)	

			custAttributes.add chestMod breatheAtt
			chestMod.breathe_controls.clavBreathe.controller = \
				clavicleBreathe.pos.controller.X_Position.controller
			chestMod.breathe_controls.chestBreathe.controller = \
				boneBreatheR.pos.controller.X_Position.controller = \
				boneBreatheL.pos.controller.X_Position.controller
			chestMod.breathe_controls.bellyBreathe.controller = \
				boneBreatheBelly.pos.controller.X_Position.controller	
		) -- if doBreathe == true then
	
		
		return #(hipControl, chestControl, torsoControl, spineHipControl, spineLowControl, spineMidControl, spineUpControl, spineChestControl)
	)
)

SSR_part = SSR_part()