----------------------------------------------------------------------------------------------------------------
--
--		Super script collection to create a simple and 
--		elegant character rig
--
--		SuperRune 2008-2013
--		rune@superrune.com
--
--		I humbly stand on the shoulders of these guys:
--		Paul Neale, Jason Schleifer and Paul Thuriot.
--
----------------------------------------------------------------------------------------------------------------

-- Global variables
global SSR_gen, SSR_loc, SSR_rig, SSR_part, SSR_skin, SSR_adv --function structs
global SSR_floater --this window
global SSR_charName --name of character
global SSR_advState = false

-- Default rig colours
global superRigRC = [0, 200, 0]
global superRigCC = [200, 200, 0]
global superRigLC = [200, 0, 0]

-- Additional files for the rig setup
fileIn "functions/superRig_generalFunctions_v1.0.ms"
fileIn "functions/superRig_locatorFunctions_v1.1.ms"
fileIn "functions/superRig_rigFunctions_v1.2.ms"
fileIn "functions/superRig_skinFunctions_v1.0.ms"
				
if doesFileExist (getFilenamePath (getSourceFileName()) + "functions/superRig_advancedFunctions_v1.1.ms") == true then 
( 
	SSR_gen.verbose "Advanced functions enabled."
	SSR_advState = true
	fileIn "functions/superRig_advancedFunctions_v1.1.ms"
)

----------------------------------------------------------------------------------------------------------------
--	Rollouts and UI functions
----------------------------------------------------------------------------------------------------------------
	-- Default character name
	if SSR_charName == undefined then 
	(
		local foundChar = false
		-- Se if there are any root controllers or locator top nodes in the scene
		for o in $objects do
		(
			if (matchPattern o.name pattern:"*_Locators_TOP" ) == true or (matchPattern o.name pattern:"*_root_CTL") == true then
			(
				SSR_gen.verbose ("Found named character in scene: " + o.name)
				SSR_charName = (filterString o.name "_")[1]
				foundChar = true
			)
		)
		if foundChar == false then SSR_charName = "Jack"
	)
	-----------------------------------------------------------------------------
	-- Rig Helpers UI
	-----------------------------------------------------------------------------
	rollout superRigMain_UI "Main rollout"
	(
		on superRigMain_UI rolledUp state do
		(
			if state then 
				SSR_floater.size = SSR_floater.size + [0,220]
			else 
				SSR_floater.size = SSR_floater.size + [0,-220]
		)
		
		bitmap rigLogo "" filename:"images/superrig_logo.bmp"
		
		edittext editCharName "Character Name: " text:SSR_charName offset:[0, 10]

		group "" 
		(
			button createLocators 	"Create Locators"	width:150 offset:[0,-6]
			spinner rigSize 		"Height: " 		range:[0,10000,(if units.SystemType == #centimeters then 180 else 70.8661)] fieldwidth:60 type:#worldunits offset:[30,1] across:2
			button fetchSize		"Fetch" 		offset:[16,-2] tooltip:"Fetch height from selected object."
		)
		
		button buildRig 		"Build Rig"	 	width:150
		
		on editCharName changed newText do (
			local oldName = SSR_charName
			local topNode = execute ("$'" + SSR_charName + "_Locators_TOP'")
			
			SSR_charName = SSR_rig.cleanName newText
			editCharName.text = SSR_charName
			SSR_gen.renameLayer (oldName + " Rig Locators") (SSR_charName + " Rig Locators")
			
			if topNode != undefined then topNode.name = SSR_charName + "_Locators_TOP"
		)
		
		/*on editCharName entered newText do (
			local oldName = SSR_charName
			local topNode = execute ("$'" + SSR_charName + "_Locators_TOP'")
			SSR_charName = SSR_rig.cleanName newText
			editCharName.text = SSR_charName
			SSR_gen.renameLayer (oldName + " Rig Locators") (SSR_charName + " Rig Locators")
			if topNode != undefined then topNode.name = SSR_charName + "_Locators_TOP"
		)*/
		
		on fetchSize pressed do
		(
			if $selection.count != 0 then rigSize.value = $.max.z - $.min.z
			try (SSR_loc.resizeLocators SSR_charName newSize:($.max.z - $.min.z)) catch()
		)
		
		on rigSize changed newSize do
		(
			try (SSR_loc.resizeLocators SSR_charName newSize:newSize)
			catch()
		)
		
		on createLocators pressed do
		(
			--try (
				local fileName = (getDir #userScripts) + "\\superSimpleRig\\rigs\\human_skeleton_180cm.rig"
				
				local oldName = SSR_loc.loadLocators fileName SSR_charName helperSize:1.0 charSize:(superRigMain_UI.rigSize.value)
				
				local loadTop = execute ("$'" + oldName + "_Locators_TOP'")
				loadTop.name = SSR_charName + "_Locators_TOP"
				SSR_gen.renameLayer (oldName + " Rig Locators") (SSR_charName + " Rig Locators")
				
				SSR_gen.verbose ("Loaded preset " + oldName + " renamed to " + SSR_charName)
			--)
			--catch SSR_gen.verbose (getCurrentException())
		)
		
		on buildRig pressed do
		(
			local doBreak = false
			-- Store current settings in rig
			try (SSR_loc.storeGUI SSR_charName) catch( SSR_gen.verbose "Problems saving rig data.")
			
			-- Is there a locator root for the named character?
			if (execute ("$'" + SSR_charName + "_Locators_TOP'")) == undefined then 
			(
				doBreak = true
				messageBox "You need to create locators before you can build the rig." title:"Oops!" beep:true
			)
			
			-- Is there a rig here already? Ask to delete
			if (execute ("$'" + SSR_charName + "_root_CTL'")) != undefined then
			(
				local doClear = queryBox ("About to delete the entire rig for the character " + SSR_charName + "!\nThis operation cannot be undone, are you sure you want to continue?")
				if doClear == true then 
				(
					SSR_gen.verbose "Deleting existing rig..."
					SSR_rig.deleteRig SSR_charName
				)
				if doClear == false then doBreak = true
			)
			if doBreak == false then
			(
			--try (
				local st = timestamp()
				local opt_UI = SSR_floater.rollouts[3]
				setCommandPanelTaskMode mode:#create
				local boneSize = superRigMain_UI.rigSize.value / 100
				local partsPath = getFilenamePath( getSourceFileName() ) + "bodyparts\\"
			
				local storedCurrent = LayerManager.current
				SSR_gen.createLayer (SSR_charName + " Rig") activeLayer:true
				
				-- Rig head
				SSR_rig.rigBodyPart partsPath "Head_Standard_" SSR_charName extParams:undefined boneSize:boneSize
				
				-- Rig spine
				local spineType = filterString opt_UI.spineType.selected " "
				local spineParams = #(opt_UI.spineAlign.state)
				SSR_rig.rigBodyPart partsPath ("Spine_" + spineType[1] + "_") SSR_charName extParams:spineParams boneSize:boneSize
				
				-- Rig arms
				local armType = filterString opt_UI.armType.selected " "
				
				local armParams = #("R", opt_UI.armTwist.state, opt_UI.lowTwist.value, opt_UI.upTwist.value)
				SSR_rig.rigBodyPart partsPath ("Arm_" + armType[1] + "_") SSR_charName extParams:armParams boneSize:boneSize 
				
				armParams = #("L", opt_UI.armTwist.state, opt_UI.lowTwist.value, opt_UI.upTwist.value)
				SSR_rig.rigBodyPart partsPath ("Arm_" + armType[1] + "_") SSR_charName extParams:armParams boneSize:boneSize 
				
			 	-- Rig fingers
				SSR_rig.rigBodyPart partsPath "Digits_Standard_" SSR_charName extParams:#("R", "fingers") boneSize:boneSize
				SSR_rig.rigBodyPart partsPath "Digits_Standard_" SSR_charName extParams:#("L", "fingers") boneSize:boneSize
				
				-- Rig legs
				local legType = filterString opt_UI.legType.selected " "
				
				local legParams = #("R", opt_UI.legTwist.state, opt_UI.shinTwist.value, opt_UI.thighTwist.value)
				SSR_rig.rigBodyPart partsPath ("Leg_" + legType[1] + "_") SSR_charName extParams:legParams boneSize:boneSize 
				
				legParams = #("L", opt_UI.legTwist.state, opt_UI.shinTwist.value, opt_UI.thighTwist.value)
				SSR_rig.rigBodyPart partsPath ("Leg_" + legType[1] + "_") SSR_charName extParams:legParams boneSize:boneSize 
				
				-- Rig toes
				SSR_rig.rigBodyPart partsPath "Digits_Standard_" SSR_charName extParams:#("R", "toes") boneSize:boneSize
				SSR_rig.rigBodyPart partsPath "Digits_Standard_" SSR_charName extParams:#("L", "toes") boneSize:boneSize
				
				-- Rig extras
				--SSR_rigExtras SSR_charName boneSize:boneSize
				SSR_rig.rigBodyPart partsPath "Extras_Standard_" SSR_charName extParams:#("L") boneSize:boneSize
				
				-- Rig root
				local rootParams = #(opt_UI.rootLock.state)
				SSR_rig.rigBodyPart partsPath "Root_Standard_" SSR_charName extParams:rootParams boneSize:boneSize
				
				-- Search for and execute postRig script
				local postRigFiles = getFiles (maxFilePath + SSR_charName + "_Postrig_*.ms")
				if postRigFiles.count > 0 then SSR_gen.verbose ("Found postrig file: " + postRigFiles[postRigFiles.count] as string + ". Must be run manually.")
				--try (fileIn postRigFiles[postRigFiles.count]) -- Must find the latest version
				--catch ( SSR_gen.verbose ("Could not find post rig maxscript file for '" + SSR_charName + "'.") )
				
				-- Put skinbones and controllers into layers
				local controllers = execute ("$'" + SSR_charname + "*_CTL'")
				SSR_gen.putToLayer (SSR_gen.collectObjects (SSR_charName + "_*_skinBone_*")) (SSR_charName + " Skinning Bones")
				SSR_gen.putToLayer controllers (SSR_charName + " Controllers")
				
				-- Controller colors and resetting
				for c in controllers do SSR_rig.resetRotation c
				SSR_rig.colorControllers controllers superRigRC superRigCC superRigLC
				SSR_rig.hideLayer (SSR_charName + " Rig")
				storedCurrent.current = true
				
				-- Cleanup
				gc light:true
				local et = timestamp()
				SSR_gen.verbose ("Rig built in " +((et-st)/1000.0) as string + " seconds")
				setCommandPanelTaskMode #modify
				
				execute ("select $'" + SSR_charName + "_root_CTL")
			)
			--)
			--catch SSR_gen.verbose (getCurrentException())
		)	
	)

	
	-----------------------------------------------------------------------------
	-- Rig positioning UI and functions
	-----------------------------------------------------------------------------
	rollout superRigProportions_UI "Rig Positioning Tools"
	(
		on superRigProportions_UI rolledUp state do
		(
			if state then 
				SSR_floater.size = SSR_floater.size + [0,360]
			else 
				SSR_floater.size = SSR_floater.size + [0,-360]
		)
		
		group "Locator Position Tools"
		(
			label straightLabel 	"Straighten:" across:4 offset:[4,4]
			button straigthenElbow 	"Elbows" width:40 offset:[12,0]
			button straigthenKnee 	"Knees" width:40 offset:[9,0]
			button straigthenFeet 	"Feet" width:40 offset:[6,0]
			button symmetryRtoL 	"Mirror Right >> Left" width:150 offset:[0, 10]
			button symmetryLtoR 	"Mirror Right << Left" width:150
		)
		
		group "Create Extra Locators"
		(
			button createHead 			"Head" 		across:3 enabled:true
			button createArms 			"Arms"		enabled:true
			button createLegs 			"Legs" 		enabled:true
			button createAnimalLegs 	"Animal Legs" across:3	enabled:SSR_advState
			button createCentered 		"Tail" enabled:SSR_advState
			button createSymmetric 		"Ears" enabled:SSR_advState
		)
		
		group "Rig preset file"		
		(
			checkBox 	resize 			"Resize presets when loading" 	checked:false offset:[16,0]
			button 		loadLocators	"Load Preset" 		across:2 width:80 toolTip:"Load a preset .rig file"
			button 		saveLocators	"Save Preset"		width:80
		)
		
		button cleanOrphans		"Clean Orphaned Lines" 	width:150 offset:[0,20] toolTip:"Click to clean character orphaned lines.\nRight click to clean ALL orphaned lines."
		button killHelpers 		"Delete Locators" 	width:150 toolTip:"Click to delete character locators.\nRight click to delete ALL locators."
		
		on straigthenElbow pressed do
		(
			try (
				SSR_loc.straightenHelper  $'Locator_Arm_R_Elbow' $'Locator_Arm_R_Shoulder' $'Locator_Arm_R_Hand'
				SSR_loc.straightenHelper  $'Locator_Arm_L_Elbow' $'Locator_Arm_L_Shoulder' $'Locator_Arm_L_Hand'
			)
			catch SSR_gen.verbose (getCurrentException())
		)
		on straigthenKnee pressed do
		(
			try (
				SSR_loc.straightenHelper  $'Locator_Leg_R_Knee' $'Locator_Leg_R_Hip' $'Locator_Leg_R_Ankle'
				SSR_loc.straightenHelper  $'Locator_Leg_L_Knee' $'Locator_Leg_L_Hip' $'Locator_Leg_L_Ankle'
			)
			catch SSR_gen.verbose (getCurrentException())
		)
		on straigthenFeet pressed do
		(
			try (
				SSR_loc.averagePosition #($'Locator_Leg_R_Heel', $'Locator_Leg_R_Toe', $'Locator_Leg_R_Tip') axis:"X"
				SSR_loc.averagePosition #($'Locator_Leg_L_Heel', $'Locator_Leg_L_Toe', $'Locator_Leg_L_Tip') axis:"X"
			)
			catch SSR_gen.verbose (getCurrentException())
		)
		on symmetryRtoL pressed do
		(
			try ( SSR_loc.symmetryHelpers "R" "L" )
			catch SSR_gen.verbose (getCurrentException())
		)
		on symmetryLtoR pressed do
		(	
			try ( SSR_loc.symmetryHelpers "L" "R" )
			catch SSR_gen.verbose (getCurrentException())
		)
		
		on createHead pressed do SSR_loc.createLocators "head" superRigMain_UI.rigSize.value SSR_charName
		on createArms pressed do SSR_loc.createLocators "arms" (superRigMain_UI.rigSize.value) SSR_charName
		on createLegs pressed do SSR_loc.createLocators "legs" (superRigMain_UI.rigSize.value) SSR_charName
		on createAnimalLegs pressed do SSR_loc.createLocators "animallegs" (superRigMain_UI.rigSize.value) SSR_charName
		on createCentered pressed do SSR_loc.createExtraLocators "tail" (superRigMain_UI.rigSize.value) SSR_charName symmetric:false
		on createSymmetric pressed do SSR_loc.createExtraLocators "ear" (superRigMain_UI.rigSize.value) SSR_charName symmetric:true
		
		on loadLocators pressed do 
		( 
			try (
				local rigPath = maxFilePath + "rigs\\"
				if maxFilePath == "" then (rigPath = getFilenamePath( getSourceFileName() ) + "rigs\\")
				
				local fileName = GetOpenFileName caption:"Select A Rig File:" \
					types:"Rig(*.rig)|*.rig|All|*.*|" \
					fileName:rigPath
				if fileName != undefined then
				(
					print ("resize state: " + resize.state as string)
					if resize.state == false then SSR_charName = SSR_loc.loadLocators fileName SSR_charName helperSize:1.0 charSize:0
					else SSR_charName = SSR_loc.loadLocators fileName SSR_charName helperSize:1.0 charSize:(superRigMain_UI.rigSize.value)

					superRigMain_UI.editCharName.text = SSR_charName
					
					-- Add: set create options to match file presets
				)
			)
			catch  SSR_gen.verbose (getCurrentException() )
		)
		on saveLocators pressed do
		(
			try (
				SSR_loc.storeGUI SSR_charName
				
				local rigPath = maxFilePath
				if maxFilePath == "" then (rigPath = getFilenamePath( getSourceFileName() ) + "rigs\\")
				
				local fileName = GetSaveFileName caption:"Enter Rig Filename:" \
					types:"Rig(*.rig)|*.rig|" \
					fileName:(rigPath + SSR_charName + ".rig")
				if fileName != undefined then
				(
					SSR_loc.saveLocators fileName SSR_charName
				)
			)
			catch SSR_gen.verbose (getCurrentException())
		)
			
		on cleanOrphans pressed do SSR_loc.cleanLines SSR_charName
		on cleanOrphans rightClick do SSR_loc.cleanLines ""
		on killHelpers pressed do SSR_loc.killLocators SSR_charName
		on killHelpers rightClick do SSR_gen.deleteLayer "* Rig Locators" deleteContent:true
	)
	
	
	-----------------------------------------------------------------------------
	--
	-- 	Rig face setup
	--
	-----------------------------------------------------------------------------
	rollout superRigFace_UI "Face Setup Tools"
	(
		on superRigFace_UI rolledUp state do
		(
			if state then 
				SSR_floater.size = SSR_floater.size + [0,70]
			else 
				SSR_floater.size = SSR_floater.size + [0,-70]
		)
		group "Placeholder"
		(
			button placeholder "Coming soon!"
		)
	)
	
	-----------------------------------------------------------------------------
	--
	-- 	Options for the rig
	--
	-----------------------------------------------------------------------------
	rollout superRigOptions_UI "Rig Creation Options"
	(
		on superRigOptions_UI rolledUp state do
		(
			if state then 
				SSR_floater.size = SSR_floater.size + [0,320]
			else 
				SSR_floater.size = SSR_floater.size + [0,-320]
		)
		
		--	Spine rig UI and functions
		group "Spine Options"
		(
			dropDownList spineType "" width:130 across:2 offset:[15,0]
			button refreshSpine "R" offset:[25,0]
			checkBox spineAlign "Align Spine Topnode to World" checked:true offset:[16,0]
		)
		on refreshSpine pressed do SSR_rig.fillDropdown (getFilenamePath( getSourceFileName() ) + "bodyparts\\") "Spine" superRigOptions_UI.spineType
		
		--	Arm rig UI and functions
		group "Arm Options"
		(
			dropDownList armType "" width:130 across:2 offset:[15,0]
			button refreshArm 	"R" offset:[25,0]
			checkBox armTwist 	"Add Twist Bones" checked:true offset:[16,0]
			spinner lowTwist 	"Under " 	range:[3,20,3] 	fieldwidth:24 type:#integer offset:[-98,0]
			spinner upTwist 	"Over " 		range:[3,20,3] 	fieldwidth:24 type:#integer offset:[-16,-20]
		)
		on refreshArm pressed do SSR_rig.fillDropdown (getFilenamePath( getSourceFileName() ) + "bodyparts\\") "Arm" superRigOptions_UI.armType
		on armTwist changed newstate do	lowTwist.enabled = upTwist.enabled = newstate
		
		--	Leg rig UI and functions
		group "Leg Options"
		(
			dropDownList legType "" width:130 across:2 offset:[15,0]
			button refreshLeg 	"R" offset:[25,0]
			checkBox legTwist 	"Add Twist Bones" checked:true offset:[16,0]
			spinner shinTwist 	"Shin " 	range:[3,20,3] 	fieldwidth:24 type:#integer offset:[-98,0]
			spinner thighTwist 	"Thigh " 	range:[3,20,3] 	fieldwidth:24 type:#integer offset:[-16,-20]
		)
		on refreshLeg pressed do SSR_rig.fillDropdown (getFilenamePath( getSourceFileName() ) + "bodyparts\\") "Leg" superRigOptions_UI.legType
		on legTwist changed newstate do shinTwist.enabled = thighTwist.enabled = newstate
		
		group "Root options"
		(
			checkBox rootLock "Add Lock Controls" checked:false offset:[16,0]
		)
		
		function doState =
		(
			upTwist.enabled = lowTwist.enabled = armTwist.enabled
			thighTwist.enabled = shinTwist.enabled = legTwist.enabled
			return true
		)
	)

	-----------------------------------------------------------------------------
	--
	-- 	Post rigging, skinning etc
	--
	-----------------------------------------------------------------------------
	rollout superPostRig_UI "Rig Skinning Tools"
	(
		on superPostRig_UI rolledUp state do
		(
			if state then 
				SSR_floater.size = SSR_floater.size + [0,100]
			else 
				SSR_floater.size = SSR_floater.size + [0,-100]
		)
		button preCheckTest "Pre Flight Check" width:150 enabled:SSR_advState
		
		group "Proxy Cut Mesh"
		(
			button skinPrecut 	"Cut Mesh" width:80 across:2 
			button killCuts		"Kill Cuts" width:80
		)
		
		on preCheckTest pressed do 
		(
			for o in $selection do SSR_adv.rigPreFlight o prefix:SSR_charName strict:true warnings:true
		)
		
		on skinPrecut pressed do SSR_skin.cutObjects SSR_charName doPrep:true
		on killCuts pressed do delete ( execute ("$'" + SSR_charName + "_proxy*'") )
	)
	
	-----------------------------------------------------------------------------
	--
	-- Extra Rig UI and functions
	--
	-----------------------------------------------------------------------------
	rollout superRigExtras_UI "Extra Rig Tools"
	(
		on superRigExtras_UI rolledUp state do
		(
			if state then 
				SSR_floater.size = SSR_floater.size + [0,260]
			else 
				SSR_floater.size = SSR_floater.size + [0,-260]
		)
		
		group "Controller Colours"
		(
			label lbl01 "Left" across:3 offset:[-8, 0]
			label lbl02 "Center" align:#center
			label lbl03 "Right" offset:[8, 0]
			colorpicker leftCol "" color:superRigLC align:#left across:3
			colorpicker centerCol "" color:superRigCC align:#center
			colorpicker rightCol "" color:superRigRC align:#right
			button recolor "Update controller colors" width:150
		)
		
		on recolor pressed do
		(
			local controllers = execute ("$'" + SSR_charname + "*_CTL'")
			SSR_rig.colorControllers controllers superRigRC superRigCC superRigLC
		)
		
		on rightCol changed newCol 		do superRigRC = newCol
		on centerCol changed newCol 	do superRigCC = newCol
		on leftCol changed newCol 		do superRigLC = newCol
		
		group "Single Joint Setup"
		(
			button singleRot "Rotation Controller" width:150 enabled:SSR_advState
			button singlePosRot "Pos Rot Controller" width:150 enabled:SSR_advState
			button linearPos "Linear Pos Controller" width:150 enabled:SSR_advState
		)
		button renameRig "Rename Rig" width:150
		button deleteRig "Delete Rig" width:150
		
		on singleRot pressed do
		(
			try SSR_adv.singleRotCon SSR_charName
			catch SSR_gen.verbose (getCurrentException())
		)
		
		on singlePosRot pressed do
		(
			try SSR_adv.singlePosRotCon SSR_charName
			catch SSR_gen.verbose (getCurrentException())
		)
		
		on linearPos pressed do
		(
			try SSR_adv.linearPosCon SSR_charName
			catch SSR_gen.verbose (getCurrentException())
		)
		
		on renameRig pressed do SSR_gen.renameRig SSR_charName
		
		on deleteRig pressed do 
		(
			local doClear = queryBox ("About to delete the entire rig for the character " + SSR_charName + "!\nThis operation cannot be undone, are you sure you want to continue?")
			if doClear == true then 
			(
				SSR_rig.deleteRig SSR_charName
			)
		)
	)

	
function SSR_buildGUI =
(
	-- See if 'PEN_Attribute_Holder 2'() is installed
	local PEN_exists = classOf 'PEN_Attribute_Holder 2'
	if PEN_exists == UndefinedClass then
	(
		PEN_string = "You do not have PEN Attribute Holder installed.\n" 
		PEN_string += "You need it to build your rigs!\n\n"
		PEN_string += "The script will now attempt to open Paul Neale's site.\n"
		PEN_string += "Follow his instructions to install the Attribute Holder."
		
		messageBox PEN_string title:"Almost there!" beep:true
		(dotnetclass "System.Diagnostics.Process").Start "http://paulneale.com/scripts/penAttributeHolder/penAttributeHolder.htm"
	) else (	
		local SSR_windowPos = [100,100]

		-- Open the GUI
		if SSR_floater != undefined then 
		(
			SSR_windowPos = SSR_floater.pos
			closeRolloutFloater SSR_floater
		)
		SSR_floater = newRolloutFloater ("Super Simple Rig " + (filterString (getFilenameFile(getSourceFileName())) "_")[3] as string) 220 330 SSR_windowPos.x SSR_windowPos.y
		
		addRollout superRigMain_UI 			SSR_floater
		addRollout superRigProportions_UI 	SSR_floater rolledUp:true
		--addRollout superRigFace_UI			SSR_floater rolledUp:true
		addRollout superRigOptions_UI 		SSR_floater rolledUp:true
		addRollout superPostRig_UI			SSR_floater rolledUp:true
		addRollout superRigExtras_UI 		SSR_floater rolledUp:true
			
		-- Fill dropdowns with rig parts
		SSR_rig.fillDropdown (getFilenamePath( getSourceFileName() ) + "bodyparts\\") "Spine" superRigOptions_UI.spineType
		SSR_rig.fillDropdown (getFilenamePath( getSourceFileName() ) + "bodyparts\\") "Arm" superRigOptions_UI.armType
		SSR_rig.fillDropdown (getFilenamePath( getSourceFileName() ) + "bodyparts\\") "Leg" superRigOptions_UI.legType
		
		-- If there's a head locator for this character, use it as height
		try (
			local locatorHead = (SSR_loc.collectLocators SSR_charName "Locator_Head_HeadTop")[1]
			superRigMain_UI.rigSize.value = locatorHead.pos.z )
		catch ()
			
		-- if a locator top is found in the scene, fetch the gui settings
		SSR_loc.setGUI SSR_charName	
	)
)

SSR_buildGUI()

