-----------------------------------------------------------------------------
--
--		Super Character Rig
--		Character Select Window
--
--		v0.5: 	First version
--		v0.6: 	Added right click as selection switch
--		v0.7: 	Reordered GUI, added reset, filter and transform buttons
--		v0.8: 	More extra functions, clear animation and offset keys
--		v0.9: 	Digits and FK/IK arm buttons
--				Temporary key all button
--		v1.1:	Button for IK/FK switch
--		v1.2:	UI tweaks, key selected button
--
-----------------------------------------------------------------------------

-- Global variables
global SSR_gen, SSR_gui, SSR_adv --function structs
global SSR_advState = false
global SSR_curSelChar

-- Additional files for the rig setup
fileIn "functions/superRig_generalFunctions_v1.0.ms"
fileIn "functions/superRig_guiFunctions_v1.0.ms"
if doesFileExist (getFilenamePath (getSourceFileName()) + "functions/superRig_advancedFunctions_v1.0.ms") == true then 
( 
	SSR_gen.verbose "Advanced functions found."
	SSR_advState = true
	fileIn "functions/superRig_advancedFunctions_v1.0.ms"
)

--function buildSelectGUI = (

	-- Remove existing dialog, create a new character list
	charList_UI = SSR_gui.listCharacters()

	rollout superRigSelector_UI ("Super Character Selector " + (filterString (getFilenameFile(getSourceFileName())) "_")[3] as string )
	(
		-- Character name select buttons
		label lbl01 "Character: " align:#left pos:[6,6]
		dropDownList charDropList "" items:charList_UI width:140 offset:[50, -22]
		button updateList "Update List" width:100 offset:[100,-27]
		
		-- Character rig select buttons
		local vis = false
		local offset=24
		bitmap rigLogo "" filename:"images/superrig_chartemplate.bmp" offset:[0,-24+offset]
		
		-- Head
		button pickHead 	"Head"  		pos:[140,10+offset] width:40 height:18 visible:vis
		button pickNeck 	"Neck"  		pos:[140,40+offset] width:40 height:18 visible:vis
		button pickLookat 	"Lookat"  		pos:[250,10+offset] width:40 height:18 visible:vis
		button pickREye 	"R"  			pos:[229,10+offset] width:20 height:18 visible:vis
		button pickLEye 	"L"  			pos:[291,10+offset] width:20 height:18 visible:vis
		button pickEyes		"Eyes"			pos:[10,10+offset] width:40 height:18 visible:vis
		button pickFace		"Face"			pos:[10,30+offset] width:40 height:18 visible:vis
		
		-- Spine
		button pickChest	"Chest"  		pos:[140,106+offset] width:40 height:18 visible:vis
		button pickBack		"Back"	  		pos:[140,128+offset] width:40 height:18 visible:vis
		button pickHip		"Hip"		  	pos:[140,150+offset] width:40 height:18 visible:vis
		button pickTorso	"Torso"	  		pos:[140,180+offset] width:40 height:18 visible:vis
		
		-- Right Arm
		button pickRClav	"Clav"			pos:[120, 80+offset] width:30 height:18 visible:vis
		button pickRShoul	"Shoulder"		pos:[72, 80+offset] width:46 height:18 visible:vis
		button pickRElbow	"Elbow"			pos:[70, 120+offset] width:36 height:18 visible:vis
		button pickRElbowIK	"IK"			pos:[50, 120+offset] width:18 height:18 visible:vis
		button pickRHandIK	"Hand IK"		pos:[20, 160+offset] width:46 height:18 visible:vis
		button pickRHandFK	"Hand FK"		pos:[20, 182+offset] width:46 height:18 visible:vis
		button pickRFingers	"Fingers"		pos:[20, 204+offset] width:46 height:18 visible:vis
		
		button pickRThumb	" "				pos:[75, 190+offset] width:18 height:10 visible:vis
		button pickRIndex	" "				pos:[60, 210+offset] width:10 height:18 visible:vis
		button pickRMiddle	" "				pos:[45, 210+offset] width:10 height:18 visible:vis
		button pickRRing	" "				pos:[30, 210+offset] width:10 height:18 visible:vis
		button pickRPinky	" "				pos:[15, 210+offset] width:10 height:18 visible:vis
		
		-- Left Arm
		button pickLClav	"Clav"			pos:[170, 80+offset] width:30 height:18 visible:vis
		button pickLShoul	"Shoulder"		pos:[202, 80+offset] width:46 height:18 visible:vis
		button pickLElbow	"Elbow"			pos:[214, 120+offset] width:36 height:18 visible:vis
		button pickLElbowIK	"IK"			pos:[252, 120+offset] width:18 height:18 visible:vis
		button pickLHandIK	"Hand IK"		pos:[250, 160+offset] width:46 height:18 visible:vis
		button pickLHandFK	"Hand FK"		pos:[250, 182+offset] width:46 height:18 visible:vis
		button pickLFingers	"Fingers"		pos:[250, 204+offset] width:46 height:18 visible:vis
		
		button pickLThumb	" "				pos:[223, 190+offset] width:18 height:10 visible:vis
		button pickLIndex	" "				pos:[246, 210+offset] width:10 height:18 visible:vis
		button pickLMiddle	" "				pos:[261, 210+offset] width:10 height:18 visible:vis
		button pickLRing	" "				pos:[276, 210+offset] width:10 height:18 visible:vis
		button pickLPinky	" "				pos:[291, 210+offset] width:10 height:18 visible:vis
		
		-- Right Leg
		button pickRHip		"Hip"			pos:[100, 180+offset] width:36 height:18 visible:vis
		button pickRKneeIK	"IK"			pos:[92, 270+offset] width:18 height:18 visible:vis
		button pickRKnee	"Knee"			pos:[112, 270+offset] width:36 height:18 visible:vis
		button pickRFootIK	"Foot IK"		pos:[100, 336+offset] width:40 height:18 visible:vis
		button pickRFootFK	"Foot FK"		pos:[100, 358+offset] width:40 height:18 visible:vis
		button pickRToes	"Toes"			pos:[88, 380+offset] width:30 height:18 visible:vis
		button pickRBall	"Ball"			pos:[120, 380+offset] width:30 height:18 visible:vis
		
		button pickRBig		" "				pos:[140, 380+offset] width:10 height:18 visible:vis
		button pickRSecond	" "				pos:[125, 380+offset] width:10 height:18 visible:vis
		button pickRThird	" "				pos:[110, 380+offset] width:10 height:18 visible:vis
		button pickRFourth	" "				pos:[96, 380+offset] width:10 height:18 visible:vis
		button pickRLittle	" "				pos:[80, 380+offset] width:10 height:18 visible:vis
		
		-- Left Leg
		button pickLHip		"Hip"			pos:[184, 180+offset] width:36 height:18 visible:vis
		button pickLKneeIK	"IK"			pos:[208, 270+offset] width:18 height:18 visible:vis
		button pickLKnee	"Knee"			pos:[170, 270+offset] width:36 height:18 visible:vis
		button pickLFootIK	"Foot IK"		pos:[180, 336+offset] width:40 height:18 visible:vis
		button pickLFootFK	"Foot FK"		pos:[180, 358+offset] width:40 height:18 visible:vis
		button pickLToes	"Toes"			pos:[202, 380+offset] width:30 height:18 visible:vis
		button pickLBall	"Ball"			pos:[170, 380+offset] width:30 height:18 visible:vis
		
		button pickLBig		" "				pos:[170, 380+offset] width:10 height:18 visible:vis
		button pickLSecond	" "				pos:[185, 380+offset] width:10 height:18 visible:vis
		button pickLThird	" "				pos:[200, 380+offset] width:10 height:18 visible:vis
		button pickLFourth	" "				pos:[215, 380+offset] width:10 height:18 visible:vis
		button pickLLittle	" "				pos:[230, 380+offset] width:10 height:18 visible:vis
		
		-- Root
		button pickRoot		"Root"		pos:[10, 380+offset] width:40 height:18 visible:vis
		
		on charDropList selected selNum do
		(
			SSR_gui.refreshSelectIcons charDropList.selected superRigSelector_UI
			SSR_curSelChar = charDropList.selected
			SSR_gen.verbose ("GUI updated for " + charDropList.selected + ".")
		)
		
		-- Character select functions
		on pickHead 	pressed do SSR_gui.pickCTL charDropList.selected "head"
		on pickHead 	rightClick do SSR_gui.boolSelCTL charDropList.selected "head"
		on pickNeck 	pressed do SSR_gui.pickCTL charDropList.selected "neck"
		on pickNeck 	rightClick do SSR_gui.boolSelCTL charDropList.selected "neck"
		on pickLookat 	pressed do SSR_gui.pickCTL charDropList.selected "lookat"
		on pickLookat 	rightClick do SSR_gui.boolSelCTL charDropList.selected "lookat"
		on pickREye 	pressed do SSR_gui.pickCTL charDropList.selected "eye_R"
		on pickREye 	rightClick do SSR_gui.boolSelCTL charDropList.selected "eye_R"
		on pickLEye 	pressed do SSR_gui.pickCTL charDropList.selected "eye_L"
		on pickLEye 	rightClick do SSR_gui.boolSelCTL charDropList.selected "eye_L"
		on pickEyes 	pressed do SSR_gui.pickCTL charDropList.selected "eyes"
		on pickEyes 	rightClick do SSR_gui.boolSelCTL charDropList.selected "eyes"
		on pickFace 	pressed do SSR_gui.pickCTL charDropList.selected "face"
		on pickFace 	rightClick do SSR_gui.boolSelCTL charDropList.selected "face"
		
		on pickChest 	pressed do SSR_gui.pickCTL charDropList.selected "chest"
		on pickBack 	pressed do SSR_gui.pickCTL charDropList.selected "back"
		on pickHip 		pressed do SSR_gui.pickCTL charDropList.selected "hip"
		on pickTorso 	pressed do SSR_gui.pickCTL charDropList.selected "torso"
		
		on pickRClav 	pressed do SSR_gui.pickCTL charDropList.selected "R_clavicle"
		on pickRShoul 	pressed do SSR_gui.pickCTL charDropList.selected "R_shoulder"
		on pickRElbow	pressed do SSR_gui.pickCTL charDropList.selected "R_elbow"
		on pickRElbowIK	pressed do SSR_gui.pickCTL charDropList.selected "R_elbow_IK"
		on pickRHandIK	pressed do SSR_gui.pickCTL charDropList.selected "R_hand_IK"
		on pickRHandFK	pressed do SSR_gui.pickCTL charDropList.selected "R_hand_FK"
		on pickRFingers pressed do SSR_gui.pickCTL charDropList.selected "R_fingers"
			
		on pickRThumb 	pressed do SSR_gui.pickCTL charDropList.selected "R_Finger_Thumb"
		on pickRIndex 	pressed do SSR_gui.pickCTL charDropList.selected "R_Finger_Index"
		on pickRMiddle 	pressed do SSR_gui.pickCTL charDropList.selected "R_Finger_Middle"
		on pickRRing 	pressed do SSR_gui.pickCTL charDropList.selected "R_Finger_Ring"
		on pickRPinky 	pressed do SSR_gui.pickCTL charDropList.selected "R_Finger_Pinkie"
		
		on pickLClav 	pressed do SSR_gui.pickCTL charDropList.selected "L_clavicle"
		on pickLShoul 	pressed do SSR_gui.pickCTL charDropList.selected "L_shoulder"
		on pickLElbow	pressed do SSR_gui.pickCTL charDropList.selected "L_elbow"
		on pickLElbowIK	pressed do SSR_gui.pickCTL charDropList.selected "L_elbow_IK"
		on pickLHandIK	pressed do SSR_gui.pickCTL charDropList.selected "L_hand_IK"
		on pickLHandFK	pressed do SSR_gui.pickCTL charDropList.selected "L_hand_FK"
		on pickLFingers pressed do SSR_gui.pickCTL charDropList.selected "L_fingers"
			
		on pickLThumb 	pressed do SSR_gui.pickCTL charDropList.selected "L_Finger_Thumb"
		on pickLIndex 	pressed do SSR_gui.pickCTL charDropList.selected "L_Finger_Index"
		on pickLMiddle 	pressed do SSR_gui.pickCTL charDropList.selected "L_Finger_Middle"
		on pickLRing 	pressed do SSR_gui.pickCTL charDropList.selected "L_Finger_Ring"
		on pickLPinky 	pressed do SSR_gui.pickCTL charDropList.selected "L_Finger_Pinkie"
			
		on pickRHip		pressed do SSR_gui.pickCTL charDropList.selected "R_hip"
		on pickRKneeIK 	pressed do SSR_gui.pickCTL charDropList.selected "R_knee_IK"
		on pickRKnee 	pressed do SSR_gui.pickCTL charDropList.selected "R_knee"
		on pickRFootFK 	pressed do SSR_gui.pickCTL charDropList.selected "R_foot_FK"
		on pickRFootIK 	pressed do SSR_gui.pickCTL charDropList.selected "R_foot_IK"
		on pickRBall 	pressed do SSR_gui.pickCTL charDropList.selected "R_toe_FK"
		on pickRToes 	pressed do SSR_gui.pickCTL charDropList.selected "R_toes"

		on pickRBig 	pressed do SSR_gui.pickCTL charDropList.selected "R_Toe_Big"
		on pickRSecond 	pressed do SSR_gui.pickCTL charDropList.selected "R_Toe_Index"
		on pickRThird 	pressed do SSR_gui.pickCTL charDropList.selected "R_Toe_Middle"
		on pickRFourth 	pressed do SSR_gui.pickCTL charDropList.selected "R_Toe_Fourth"
		on pickRLittle 	pressed do SSR_gui.pickCTL charDropList.selected "R_Toe_Little"
		
		on pickLHip		pressed do SSR_gui.pickCTL charDropList.selected "L_hip"
		on pickLKneeIK 	pressed do SSR_gui.pickCTL charDropList.selected "L_knee_IK"
		on pickLKnee 	pressed do SSR_gui.pickCTL charDropList.selected "L_knee"
		on pickLFootFK 	pressed do SSR_gui.pickCTL charDropList.selected "L_foot_FK"
		on pickLFootIK 	pressed do SSR_gui.pickCTL charDropList.selected "L_foot_IK"
		on pickLBall 	pressed do SSR_gui.pickCTL charDropList.selected "L_toe_FK"
		on pickLToes 	pressed do SSR_gui.pickCTL charDropList.selected "L_toes"
			
		on pickLBig 	pressed do SSR_gui.pickCTL charDropList.selected "L_Toe_Big"
		on pickLSecond 	pressed do SSR_gui.pickCTL charDropList.selected "L_Toe_Index"
		on pickLThird 	pressed do SSR_gui.pickCTL charDropList.selected "L_Toe_Middle"
		on pickLFourth 	pressed do SSR_gui.pickCTL charDropList.selected "L_Toe_Fourth"
		on pickLLittle 	pressed do SSR_gui.pickCTL charDropList.selected "L_Toe_Little"

		on pickRoot 	pressed do SSR_gui.pickCTL charDropList.selected "root"
		on pickRoot 	rightClick do SSR_gui.boolSelCTL charDropList.selected "root"
		
		-- Character right click functions
		on pickChest 	rightClick do SSR_gui.boolSelCTL charDropList.selected "chest"
		on pickBack 	rightClick do SSR_gui.boolSelCTL charDropList.selected "back"
		on pickHip 		rightClick do SSR_gui.boolSelCTL charDropList.selected "hip"
		on pickTorso 	rightClick do SSR_gui.boolSelCTL charDropList.selected "torso"
		
		on pickRClav 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_clavicle"
		on pickRShoul 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_shoulder"
		on pickRElbow	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_elbow"
		on pickRElbowIK	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_elbow_IK"
		on pickRHandIK	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_hand_IK"
		on pickRHandFK	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_hand_FK"
		on pickRFingers rightClick do SSR_gui.boolSelCTL charDropList.selected "R_fingers"
			
		on pickRThumb 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Finger_Thumb"
		on pickRIndex 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Finger_Index"
		on pickRMiddle 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Finger_Middle"
		on pickRRing 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Finger_Ring"
		on pickRPinky 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Finger_Pinkie"
		
		on pickLClav 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_clavicle"
		on pickLShoul 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_shoulder"
		on pickLElbow	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_elbow"
		on pickLElbowIK	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_elbow_IK"
		on pickLHandIK	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_hand_IK"
		on pickLHandFK	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_hand_FK"
		on pickLFingers rightClick do SSR_gui.boolSelCTL charDropList.selected "L_fingers"
			
		on pickLThumb 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Finger_Thumb"
		on pickLIndex 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Finger_Index"
		on pickLMiddle 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Finger_Middle"
		on pickLRing 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Finger_Ring"
		on pickLPinky 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Finger_Pinkie"
			
		on pickRHip		rightClick do SSR_gui.boolSelCTL charDropList.selected "R_hip"
		on pickRKneeIK 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_knee_IK"
		on pickRKnee 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_knee"
		on pickRFootFK 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_foot_FK"
		on pickRFootIK 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_foot_IK"
		on pickRBall 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_toe_FK"
		on pickRToes 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_toes"

		on pickRBig 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Toe_Big"
		on pickRSecond 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Toe_Index"
		on pickRThird 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Toe_Middle"
		on pickRFourth 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Toe_Fourth"
		on pickRLittle 	rightClick do SSR_gui.boolSelCTL charDropList.selected "R_Toe_Little"
		
		on pickLHip		rightClick do SSR_gui.boolSelCTL charDropList.selected "L_hip"
		on pickLKneeIK 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_knee_IK"
		on pickLKnee 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_knee"
		on pickLFootFK 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_foot_FK"
		on pickLFootIK 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_foot_IK"
		on pickLBall 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_toe_FK"
		on pickLToes 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_toes"
			
		on pickLBig 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Toe_Big"
		on pickLSecond 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Toe_Index"
		on pickLThird 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Toe_Middle"
		on pickLFourth 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Toe_Fourth"
		on pickLLittle 	rightClick do SSR_gui.boolSelCTL charDropList.selected "L_Toe_Little"
		
		-- Update character list
		on updateList pressed do
		(
			local charList_UI = SSR_gui.listCharacters()
			charDropList.items = charList_UI
			SSR_gui.refreshSelectIcons charDropList.selected superRigSelector_UI
		)
		
		-- Buttons on the side
		button selAll 					"selall" 	pos:[286, 276+offset] width:32 height:16
		button keyAll 					"keyall" 	pos:[286, 294+offset] width:32 height:16
		button keySel 					"keysel" 	pos:[286, 320+offset] width:32 height:16
		checkbutton selectControllers 	"filter" 	pos:[286, 346+offset] width:32 height:16
		button showControllers 			"hide" 		pos:[286, 362+offset] width:32 height:16
		button openMore					"more"		pos:[286, 388+offset] width:32 height:16
		
		-- Extra rollout buttons
		local offset=440
		group "Reset Attributes" (
			label resetLabel		"Zero:" 			pos:[10, 14+offset]
			button resetAll			"Character" 		pos:[45, 10+offset] width:55
			checkbox resetStatic 	"Don't reset IK"	pos:[200, 13+offset] width:164 checked:true
			button resetSel			"Selected" 			pos:[105, 10+offset] width:55
		)
		
		local offset=414
		group "Transform Gizmo" (
			button coordVie		"View"	pos:[10, 85+offset]		width:30
			button coordLoc		"Loc"	pos:[45, 85+offset]		width:30
			button coordPar		"Par"	pos:[80, 85+offset]		width:30
			button coordGim		"Gim"	pos:[115, 85+offset]	width:30
			button centerBox	"Toggle Center Box" pos:[186, 85+offset] 	width:120
		)
		group "Edit Keys" (
			label offsetLabel	"Offset:" 	pos:[10, 138+offset] enabled:SSR_advState
			button offsetChar	"Character"	pos:[45, 134+offset]		width:55 enabled:SSR_advState
			button offsetSel	"Selected"	pos:[105, 134+offset]		width:55 enabled:SSR_advState
			
			label clearLabel	"Clear:"	pos:[165, 138+offset] enabled:SSR_advState
			button clearChar	"Character"	pos:[195, 134+offset]		width:55 enabled:SSR_advState
			button clearSel		"Selected"	pos:[255, 134+offset]		width:55 enabled:SSR_advState
			
			radiobuttons frameRange labels:#("All", "Range", "") default:1 pos:[10,163+offset] enabled:SSR_advState
			spinner firstFrame 	"S:" range:[0,animationRange.end,animationRange.start] \
				type:#integer  fieldWidth:28 pos:[120,164+offset] enabled:false
			spinner lastFrame 	"E: " range:[0,animationRange.end,animationRange.end] \
				type:#integer fieldWidth:28 pos:[174,164+offset] enabled:false
			spinner frameOffset "Offset: " range:[-1000,1000,0] \
				type:#integer fieldWidth:28 pos:[234,164+offset] enabled:SSR_advState
		)
		group "Switches and Locks" (
			button ikSwitch	"FK/IK Switch"	pos:[10, 208+offset]	width:120
			label lockLabel "Lock root:"	pos:[150, 212+offset]
			button lockWorld "World"		pos:[204, 208+offset]	width:55
			button lockOne "1"				pos:[264, 208+offset]	width:20
			button lockTwo "2"				pos:[290, 208+offset]	width:20
		)
		
		on selAll pressed do
		(
			if charDropList.selected != undefined then
			execute ("select $'" + charDropList.selected + "_*_CTL")
		)
		
		on keyAll pressed do
		(
			if charDropList.selected != undefined then
			SSR_gui.keyCharAttributes charDropList.selected all:true
		)
		
		on keyAll rightClick do
		(
			if charDropList.selected != undefined then
			SSR_gui.keyCharAttributes charDropList.selected all:true forceAnimButton:false
		)
		
		on keySel pressed do
		(
			if charDropList.selected != undefined then
			SSR_gui.keyCharAttributes charDropList.selected all:false
		)
		
		on openMore pressed do
		(
			local curSize = GetDialogSize superRigSelector_UI
			if curSize.y == 430 then (
				local storeSel = charDropList.selection
				selWinPos = GetDialogPos superRigSelector_UI
				DestroyDialog superRigSelector_UI 
				createDialog superRigSelector_UI 320 654 pos:selWinPos lockHeight:true
				charDropList.selection = storeSel
				SSR_gui.refreshSelectIcons charDropList.selected superRigSelector_UI
			)
			if curSize.y > 430 then (
				local storeSel = charDropList.selection
				selWinPos = GetDialogPos superRigSelector_UI
				DestroyDialog superRigSelector_UI 
				createDialog superRigSelector_UI 320 430 pos:selWinPos lockHeight:true
				charDropList.selection = storeSel
				SSR_gui.refreshSelectIcons charDropList.selected superRigSelector_UI
			)				
		)
		
		on resetAll pressed do 
		(
			if charDropList.selected != undefined then
			SSR_gui.resetCharAttributes charDropList.selected all:true static:resetStatic.state
		)
		on resetSel pressed do 
		(
			if charDropList.selected != undefined then
			SSR_gui.resetCharAttributes charDropList.selected all:false static:resetStatic.state
		)
			
		on selectControllers changed state do
			if state == on then SetSelectFilter 3
			else SetSelectFilter 1
			
		on showControllers pressed do
		(
			try (
				local CTRLayerName = charDropList.selected + " Controllers"
				local CTRLayer = layermanager.getlayerfromname CTRLayerName
				CTRLayer.ishidden = not (CTRLayer.ishidden)
			) catch()
			try (
				local CTRLayerName = charDropList.selected + " Face Controllers"
				local CTRLayer = layermanager.getlayerfromname CTRLayerName
				CTRLayer.ishidden = not (CTRLayer.ishidden)
			) catch()
		)
		
		on centerBox pressed do tmGizmos.useCenterBox = not (tmGizmos.useCenterBox)
		on coordVie	pressed do toolMode.coordsys #view
		on coordLoc	pressed do toolMode.coordsys #local
		on coordPar	pressed do toolMode.coordsys #parent
		on coordGim	pressed do toolMode.coordsys #gimbal

		on frameRange changed state do
		(
			if state != 3 then 
			(
				firstFrame.enabled = false
				lastFrame.enabled = false
			) else (
				firstFrame.enabled = true
				lastFrame.enabled = true
			)
		)
		
		-- We sneak in a function that fetches the current frame range!
		function getUIFrameRange =
		(
			-- All keyframes
			local start = -100000
			local end = 10000000
			
			-- Current range
			if frameRange.state == 2 then
			(
				start = animationRange.start
				end = animationRange.end
			)
				
			-- Custom range
			if frameRange.state == 3 then
			(
				start = firstFrame.value
				end = lastFrame.value
			)
			
			return #(start, end)
		)
		
		on clearChar pressed do
		(
			local doClear = queryBox "This will clear all animation data!\nAre you sure you want to continue?"
			if doClear == true then 
			(
				if $selection.count == 0 then execute ("select $'" + charDropList.selected + "_root_CTL'")
				firstFrame.value
				SSR_adv.clearKeys "character" range:(getUIFrameRange()) debug:false 
			)
		)
		
		on clearSel pressed do
		(
			local doClear = queryBox "This will clear all animation data\non the selected controllers!\n\nAre you sure you want to continue?"
			if doClear == true then SSR_adv.clearKeys "selected" range:(getUIFrameRange()) debug:false
		)
		
		on offsetChar pressed do 
		(
			if $selection.count == 0 then execute ("select $'" + charDropList.selected + "_root_CTL'")
			SSR_adv.offsetKeys "character" offset:frameOffset.value range:(getUIFrameRange()) debug:false
		)
		
		on offsetSel pressed do 
		(
			SSR_adv.offsetKeys "selected" offset:frameOffset.value range:(getUIFrameRange()) debug:false
		)
		
		on ikSwitch pressed do try( SSR_gui.ikSwitch charDropList.selected ) catch( getCurrentException() )
		on lockWorld pressed do try ( SSR_gui.lockRoot charDropList.selected 0 ) catch( getCurrentException() )
		on lockOne pressed do try ( SSR_gui.lockRoot charDropList.selected 1 ) catch( getCurrentException() )
		on lockTwo pressed do try ( SSR_gui.lockRoot charDropList.selected 2 ) catch( getCurrentException() )		
	)

	
	
/*	local superRigPos
	try (superRigPos = GetDialogPos superRigSelector_UI) catch ()
	print (GetDialogPos superRigSelector_UI)
	if superRigPos == undefined then superRigPos = [100,100]
	DestroyDialog superRigSelector_UI */-- This will not work. Why?

	-- Open the GUI
	createDialog superRigSelector_UI 320 430 pos:[100,100] lockHeight:true
	if SSR_curSelChar != undefined then SSR_gui.setSelection superRigSelector_UI.charDropList SSR_curSelChar
	SSR_gui.refreshSelectIcons superRigSelector_UI.charDropList.selected superRigSelector_UI
--)

--buildSelectGUI()