-----------------------------------------------------------------------------
--
--		Animation clip save and load
--
--		ToDo:
--		- Make sure these special cases work:
--		  When loading no keys on static tracks
--		  or when loading static tracks on tracks with keys
--		(if loaded track has keys, add buffer frame at start and end)
--		- Save and Load poses
--		- Don't key tracks that have no keys loaded and keys pre-loading
--
-----------------------------------------------------------------------------

-- Global variables
global SSR_gen, SSR_adv --function structs
global SSR_advState = false
global SSR_clipData = undefined
global superRigSave_floater

-- Additional files for the rig setup
fileIn "functions/superRig_generalFunctions_v1.0.ms"
if doesFileExist (getFilenamePath (getSourceFileName()) + "functions/superRig_advancedFunctions_v1.1.ms") == true then 
( 
	SSR_gen.verbose "Advanced functions found."
	SSR_advState = true
	fileIn "functions/superRig_advancedFunctions_v1.1.ms"
)

-----------------------------------------------------------------------------
-- Build GUI
-----------------------------------------------------------------------------
function superRigSave_GUI =
(	
	-----------------------------------------------------------------------------
	-- Save animation GUI
	-----------------------------------------------------------------------------
	rollout superRigSave_UI "Save Animation"
	(
		group "Save Animation Data" 
		(
			button saveAll "Scene" width:80 across:3 enabled:SSR_advState
			button saveChar "Character" width:80 enabled:SSR_advState
			button saveCtrl "Selected" width:80 enabled:SSR_advState
			checkBox clearFolderFirst "Clear existing files from clip folder" checked:true offset:[7,0]
		)
			
		group "Frame Range"
		(
			radiobuttons frameRange labels:#("All", "Range", "") default:1 offset:[-66,0]
			spinner firstFrame "Start: " range:[0,animationRange.end,animationRange.start] type:#integer  fieldWidth:28 offset:[-80,-19] enabled:false
			spinner lastFrame "End: " range:[0,animationRange.end,animationRange.end] type:#integer fieldWidth:28 offset:[-10,-21] enabled:false
		)
		
		group "Comment"
		(
			editText commentBox text:"..." fieldWidth:264 height:32 enabled:SSR_advState
		)
		
		function getUIFrameRange =
		(
			-- All keyframes
			local start = -1000f
			local end = 100000f
			
			-- Current range
			if frameRange.state == 2 then
			(
				start = animationRange.start
				end = animationRange.end
			)
				
			-- Custom range
			if frameRange.state == 3 then
			(
				start = firstFrame.value
				end = lastFrame.value
			)
			
			return #(start, end)
		)
		
		on saveAll pressed do 
		(
			local controllerList = SSR_gen.collectControllers "*"
			SSR_adv.saveAnim commentBox.text controllerList \
			clearFolder:(clearFolderFirst.checked) range:(getUIFrameRange()) debug:false
			try (superRigLoad_UI.refreshList superRigLoad_UI.pathTxt.text) catch(getCurrentException())
		)
		on saveChar pressed do 
		(
			local controllerList = SSR_gen.collectControllers (superRigSave_floater.rollouts[3].sceneAssets.selected + "_*")
			SSR_adv.saveAnim commentBox.text controllerList \
			clearFolder:(clearFolderFirst.checked) range:(getUIFrameRange()) debug:false
			try (superRigLoad_UI.refreshList superRigLoad_UI.pathTxt.text) catch(getCurrentException())
		)
		on saveCtrl pressed do 	
		(
			local controllerList = SSR_gen.filterControllers $selection
			SSR_adv.saveAnim commentBox.text controllerList \
			clearFolder:(clearFolderFirst.checked) range:(getUIFrameRange()) debug:false
			try (superRigLoad_UI.refreshList superRigLoad_UI.pathTxt.text) catch(getCurrentException())
		)
		on saveCtrl rightclick do 	
		(
			local controllerList = SSR_gen.filterControllers $selection
			SSR_adv.saveAnim commentBox.text controllerList \
			clearFolder:(clearFolderFirst.checked) range:(getUIFrameRange()) debug:true
			try (superRigLoad_UI.refreshList superRigLoad_UI.pathTxt.text) catch(getCurrentException())
		)
		
		
		on frameRange changed state do
		(
			if state != 3 then 
			(
				firstFrame.enabled = false
				lastFrame.enabled = false
			) else (
				firstFrame.enabled = true
				lastFrame.enabled = true
			)
		)
		
		on superRigSave_UI rolledUp state do
		(
			if state then 
				superRigSave_floater.size = superRigSave_floater.size + [0,190]
			else 
				superRigSave_floater.size = superRigSave_floater.size + [0,-190]
		)
	) -- rollout superRigSave_UI "Save Animation"

	-----------------------------------------------------------------------------
	-- Load animation GUI
	-----------------------------------------------------------------------------
	try SSR_clipData = SSR_adv.fetchClipList (maxFilePath + "Clips\\") catch ()
	if SSR_clipData == undefined or SSR_clipData.count == 0 then SSR_clipData = #(#("No files", "", "...", ""))
	
	rollout superRigLoad_UI "Load Animation"
	(
		group "Load Animation Data" 
		(
			button loadAll "Scene" width:80 across:3 enabled:SSR_advState
			button loadChar "Character" width:80 enabled:SSR_advState
			button loadCtrl "Controllers" width:80 enabled:SSR_advState toolTip:"Right click for verbose debugging"
			checkBox clearRange "Clear existing range" checked:true offset:[7,0] across:2
			spinner loadOffset "Frame offset: " range:[-10000,100000,0] type:#integer fieldWidth:37 offset:[-10,1]
			button forceLoad "Force load animation file to single controller" width:264 enabled:SSR_advState
		)
			
		group "Frame Range"
		(
			radiobuttons frameRange labels:#("All", "Range", "") default:1 offset:[-80,0]
			spinner firstFrame "Start: " range:[0,animationRange.end,animationRange.start] type:#integer  fieldWidth:37 offset:[-80,-19] enabled:false
			spinner lastFrame "End: " range:[0,animationRange.end,animationRange.end] type:#integer fieldWidth:37 offset:[-5,-21] enabled:false
		)
		
		editText pathTxt "Path: " fieldWidth:150 across:4 text:(maxFilePath + "Clips\\") offset:[0,7]
		button pathLoad "..." width:16 offset:[92,5]
		button pathRefresh "Refresh" offset:[60,5] toolTip:"Right click to reset to scene path"
		button pathExplorer "E" width:16 offset:[28,5]
		listBox fileListBox items:(for o in SSR_clipData collect o[1]) height:5 selection:SSR_clipData.count
		editText commentTxt text:SSR_clipData[SSR_clipData.count][3] width:276 height:32 offset:[-3,0] readOnly:true
		
		function getUIFrameRange =
		(
			-- All keyframes
			local start = -100000
			local end = 10000000
			
			-- Current range
			if frameRange.state == 2 then
			(
				start = animationRange.start
				end = animationRange.end
			)
				
			-- Custom range
			if frameRange.state == 3 then
			(
				start = firstFrame.value
				end = lastFrame.value
			)
			
			return #(start, end)
		)
		
		on loadAll pressed do 
		(
			local controllerList = SSR_gen.collectControllers "*"
			SSR_adv.loadAnim controllerList pathTxt.text fileListBox.selection \
				doClear:clearRange.state offset:loadOffset.value range:(getUIFrameRange()) debug:false
		)
		on loadChar pressed do 
		(
			local controllerList = SSR_gen.collectControllers (superRigSave_floater.rollouts[3].sceneAssets.selected + "_*")
			SSR_adv.loadAnim controllerList pathTxt.text fileListBox.selection \
				doClear:clearRange.state offset:loadOffset.value range:(getUIFrameRange()) debug:false
		)
		on loadCtrl pressed do 
		(
			local controllerList = SSR_gen.filterControllers $selection
			SSR_adv.loadAnim controllerList pathTxt.text fileListBox.selection \
				doClear:clearRange.state offset:loadOffset.value range:(getUIFrameRange()) debug:falsee
		)
		on loadCtrl rightclick do 
		(
			local controllerList = SSR_gen.filterControllers $selection
			SSR_adv.loadAnim controllerList pathTxt.text fileListBox.selection \
				doClear:clearRange.state offset:loadOffset.value range:(getUIFrameRange()) debug:true
		)
				
		on frameRange changed state do
		(
			if state != 3 then 
			(
				firstFrame.enabled = false
				lastFrame.enabled = false
			) else (
				firstFrame.enabled = true
				lastFrame.enabled = true
			)
		)
		
		function refreshList fetchDir =
		(
			SSR_clipData = SSR_adv.fetchClipList fetchDir
			if SSR_clipData.count == 0 then SSR_clipData = #(#("No files", "", "...", ""))
			else (
				fileListBox.items = (for o in SSR_clipData collect o[1])
				fileListBox.selection = SSR_clipData.count
				commentTxt.text = SSR_clipData[SSR_clipData.count][3]
			)
		)
		
		on pathTxt entered newText do refreshList pathTxt.text
		
		on forceLoad pressed do 
		(
			local loadPath = pathTxt.text
			fileName = getOpenFileName caption:"Open Animation Clip:" types:"Clip(*.clip)" \
				fileName:loadPath historyCategory:"AnimationClips"
			if fileName != undefined then
			(
				SSR_adv.loadKeys fileName $ range:(getUIFrameRange()) debug:true
			)
		)
		
		on pathLoad pressed do
		(
			local newDir = getSavePath caption:"Select an animation clip path." initialDir:pathTxt.text
			if newDir != undefined then
			(
				pathTxt.text = newDir + "\\"
				refreshList pathTxt.text
			)
		)
		
		on pathRefresh pressed do refreshList pathTxt.text
		on pathRefresh rightClick do 
		(
			pathTxt.text = maxFilePath + "Clips\\"
			refreshList pathTxt.text
		)
		
		on pathExplorer pressed do ShellLaunch "Explorer" (pathTxt.text)
		
		on fileListBox selected itemNo do commentTxt.text = SSR_clipData[itemNo][3]
		on fileListBox doubleClicked itemNo do 
		(
			commentTxt.text = SSR_clipData[itemNo][3]
			doVerbose fileListBox.selected
		)
		
		on superRigLoad_UI rolledUp state do
		(
			if state then 
				superRigSave_floater.size = superRigSave_floater.size + [0,300]
			else 
				superRigSave_floater.size = superRigSave_floater.size + [0,-300]
		)	
	) -- rollout superRigLoad_UI "Load Animation"

	-----------------------------------------------------------------------------
	-- Replace asset GUI
	-----------------------------------------------------------------------------
	rollout replaceAsset_UI "Replace Assets"
	(
		editText assetPath "Asset Path: " fieldWidth:170 across:2 text:"P:\\Production\\Assets\\" offset:[0,7]
		button pickAssetPath "..." offset:[50,5]
		--button doReplaceAll "Update All Assets To Latest Version" width:264

		group "Replace Single Asset"
		(
			label lbl1 "Asset: " align:#left across:4
			dropDownList sceneAssets "" items:(SSR_gen.getCharArray()) width:140 offset:[-32, -3]
			button updateList "R" width:20 offset:[20,-3]
			button doReplace "Pick file" width:70 offset:[0,-3] enabled:SSR_advState --across:2 
		)
		
		on pickAssetPath pressed do
		(
			local newDir = getSavePath caption:"Select your asset folder." initialDir:assetPath.text
			if newDir != undefined then assetPath.text = newDir + "\\"
		)
		
		on updateList pressed do
		(
			sceneAssets.items = SSR_gen.getCharArray()
		)
		
		on doReplaceAll pressed do
		(
			local replaceCount = 0
			
			-- Save out scene animation
			SSR_adv.saveSceneAnim ("Asset Replace on All") "all" saveScene:false increment:false debug:false
			
			for c in sceneAssets.items do
			(
				local charName = c
				
				-- Find recent asset
				SSR_gen.verbose ("Finding most recent version of asset '" + c + "'")
				local theFiles = SSR_adv.prefixFiles c assetPath.text
				
				if theFiles == false then
				(
					SSR_gen.verbose ("No asset files found in path.")
				) else (
					local fileName = theFiles[theFiles.count]
					SSR_gen.verbose ("Most recent version of '" + c + "' is '" + fileName + "'.")
					
					-- !TODO See if the found asset is newer than the one in the scene
					
					SSR_gen.deleteLayer (charName + " *") deleteContent:true
					charObjs = execute ("select $'" + charName + "_*'")
					delete $selection
					
					local mergeResult = mergeMaxFile fileName #select #promptDups #promptMtlDups #promptReparent quiet:true
					SSR_gen.verbose ("Merge: " + mergeResult as string)
						
					replaceCount = replaceCount + 1
				)
			)	
					
			if replaceCount > 0 then 
			(
				local clipPath = (maxFilePath + "Clips\\")
				local pathIndex = (getDirectories (clipPath + "*")).count
				
				SSR_adv.clearKeys "selected" range:(getUIFrameRange()) debug:false
				SSR_adv.loadAnim "all" clipPath pathIndex \
					matchName:true doClear:false debug:false

				messageBox "Saving and loading animation can make 3ds Max unstable.\nYou are advised to save your work and restart 3ds Max." title:"A very good time to restart."
			)
		)
		
		on doReplace pressed do
		(
			local charName = sceneAssets.selected
				
			-- Warn about objects that will be deleted
			local letsDoThis = queryBox ("Warning! Save your file!\nThis will delete everything (layers, objects)\nthat starts with '" + charName +"'\n\nAre you sure you want to continue?")
			
			if letsDoThis == true then
			(
				--Get new asset file
				fileName = getOpenFileName caption:"Open Asset File:" types:"3ds Max File(*.max)" \
					fileName:(assetPath.text) historyCategory:"MaxFiles"
				if fileName != undefined then
				(
					SSR_gen.verbose ("Replacing asset '" + charName + "' with assets from: " + fileName)
					SSR_gen.deleteAsset charName
					
					-- For some reason, this does not update correctly. Why?
					mergeMaxFile fileName #select #promptDups #promptMtlDups #promptReparent 
				
					local clipPath = (maxFilePath + "Clips\\")
					local pathIndex = (getDirectories (clipPath + "*")).count
					
					-- Load the animation back in
					local controllerList = SSR_gen.collectControllers (charName + "_*")
					SSR_adv.loadAnim controllerList clipPath (superRigSave_floater.rollouts[2].fileListBox.selection) doClear:true debug:true
							
					-- Ask to restart max
					messageBox "Saving and loading animation can make 3ds Max unstable.\nThis script recommends you save your work and restart 3ds Max." title:"A good time to restart."
				)
			) else (
				SSR_gen.verbose ("Cancelled replace of " + charName + ".")	
			) -- if letsDoThis == true then
		) -- on doReplace pressed do
		
		on replaceAsset_UI rolledUp state do
		(
			if state then 
				superRigSave_floater.size = superRigSave_floater.size + [0,90]
			else 
				superRigSave_floater.size = superRigSave_floater.size + [0,-90]
		)	
	) -- rollout replaceAsset_UI "Replace Asset"
	
	
	local superRigSavePos = [100,100]

	-- Open the GUI
	if superRigSave_floater != undefined then 
	(
		superRigSavePos = superRigSave_floater.pos
		closeRolloutFloater superRigSave_floater
	)
	superRigSave_floater = newRolloutFloater ("Super Rig Saver " + (filterString (getFilenameFile(getSourceFileName())) "_")[3] as string) 312 655 superRigSavePos.x superRigSavePos.y
	addRollout superRigSave_UI superRigSave_floater
	addRollout superRigLoad_UI superRigSave_floater
	addRollout replaceAsset_UI superRigSave_floater
)

superRigSave_GUI()
