-- Super Simple Rig installer

(
	try ( destroyDialog SSR_rollout ) catch ()
	local prevInstall = false
	local findInstall = getDirectories ((getDir #userScripts) + "\\superSimpleRig")
	if findInstall.count != 0 then prevInstall = true
	
	rollout SSR_rollout "Super Simple Rig Installer"
	(
		-- Check if there is an existing super rig folder
		label inst1 "Clicking the install button will install Super Simple Rig" offset:[0,20]
		label inst2 "in your 3ds Max user scripts folder."
		label inst3 "Warning: A previous install has been found at 'scripts/superSimpleRig'" offset:[0,20] visible:prevInstall
		label inst4 "Make sure you have no personal files in this folder before installing." visible:prevInstall
		button unInstall "Remove existing installation" offset:[0,10] visible:prevInstall
		
		button doInstall "Install Super Simple Rig" offset:(if previnstall then [0,30] else [0,-50])
		
		function verbose textString = ( format "-- %\n" textString )
		
		function moveFiles sourceFolder destFolder filePattern deleteOriginal:true =
		(
			local fileList = getFiles (sourceFolder + "\\" + filePattern)
			for f in fileList do
			(
				copyFile f (destFolder + "//" + (filenameFromPath f))
				if deleteOriginal == true then deleteFile f
				
				verbose ("Moving file '" + (filenameFromPath f) + "' from: '" + sourceFolder + "' to: '" + destFolder + "'")
			)
			return true
		)
			
		function moveFolder sourceFolder destFolder filePattern deleteOriginal:true =
		(
			verbose ("Moving folder: '" + sourceFolder + "' to: '" + destFolder + "'")
			
			-- If target folder does not exist, create it.
			makeDir destFolder all:true
			verbose ("Creating folder: '" + destFolder + "'")
			
			-- Move files in this folder
			moveFiles sourceFolder destFolder "*.*" deleteOriginal:deleteOriginal
			
			-- Process sub-folders
			local subFolders = getDirectories (sourceFolder + "\\*")
			for s in subFolders do
			(
				local subFolder = filterString s "\\"
				verbose ("Moving folder '" + subFolder[subFolder.count] + "' in '" + sourceFolder + "'")
				makeDir (destFolder + "\\" + subFolder[subFolder.count]) all:true
				moveFiles s (destFolder + "\\" + subFolder[subFolder.count]) "*.*" deleteOriginal:deleteOriginal
			)
			return true
		)
		
		function cleanExisting =
		(
			local scriptDest = getDir #userScripts
			local macrosDest = getDir #userMacros
			local iconsDest = getDir #userIcons
			
			-- Delete existing files
			try ( 
				dosCommand ("rmDir \"" + scriptDest + "\\superSimpleRig\" /s /q")
				verbose ("Deleting folder: " + scriptDest + "\\superSimpleRig")
			) catch print ("Could not remove old script files.")
			try (
				deleteFile (macrosDest + "\\superrune_SSR.mcr")
				verbose ("Deleting file: " + macrosDest + "\\superrune_SSR.mcr")
				deleteFile (iconsDest + "\\superrune_SSR_24i.bmp")
				verbose ("Deleting file: " + iconsDest + "\\superrune_SSR_24i.bmp")
				deleteFile (iconsDest + "\\superrune_SSR_16i.bmp")
				verbose ("Deleting file: " + iconsDest + "\\superrune_SSR_16i.bmp")
			) catch print ("Could not remove old ui files.")
			
		)
		
		on unInstall pressed do
		(
			try (
				cleanExisting()
				destroydialog SSR_Rollout
				messagebox "Super Simple Rig should be removed from your system. Check the Listener for any errors." title:"Uninstallation Complete"
			)
			catch print ("Error when uninstalling:\n" + getCurrentException() )
			verbose "Super Simple Rig uninstall complete."
		)
		
		on doInstall pressed do
		(
			verbose "Starting install."
			
			local scriptDest = getDir #userScripts
			local macrosDest = getDir #userMacros
			local iconsDest = getDir #userIcons
			
			local installSource = getFilenamePath(getSourceFilename()) + "_superSimpleRig_install"			
			
			-- Clean existing files
			cleanExisting()
				
			-- Do a new install
			try (
				-- move files into their locations
				moveFiles (installSource + "\\usericons") iconsDest "*.bmp"
				moveFiles (installSource + "\\usermacros") macrosDest "*.mcr"
				moveFolder (installSource + "\\superSimpleRig") (scriptDest + "\\superSimpleRig") "*.*"
				
				destroydialog SSR_Rollout
				messagebox "Please restart 3ds Max\n\nAfter restart, you can find Super Simple Rig under the 'SuperRune' category inside the 'Customize User Interface...' menu." title:"Installation Complete"
			) 
			catch print ("Installation error:\n" + getCurrentException() )
			verbose "Super Simple Rig install complete."
		)		 
	)
	if prevInstall == true then createDialog SSR_rollout 400 220
	else createDialog SSR_rollout 400 140
)